/*!
 * @file
 * @brief カレンダー本体プログラム
 * @author y_okamon
 * @version 1.0(2007.04.25) 初版
 * @version 1.1(2008.10.07) <br>
 * 3ヶ月分格納配列の型変更に伴う仕様変更<br>
 * (1) 3ヶ月分のカレンダーを格納する関数make_calendarを作成<br>
 * (2) 祝日名と日付を入れる関数make_holidayを作成<br>
 * (3) (1)(2)より関数make_calendarを削除<br>
 */

/* ここからヘッダファイルの宣言 */
#include<stdio.h>
#include<string.h>
#include "number_set.h"

/* ここから関数のプロトタイプの宣言 */
void reset(int cal_buf[COL_CAL][WEEK_ROW][WEEK_COL]);

int judge_month_1989(int year, int month, char holiday_name[HOLY_MAX][HOLY_NAME], int holiday_num[HOLY_MAX]);

/*!
 * <b>機能</b><br>
 * 年が閏年かどうかを調べる<br><br>
 * 
 * @param year 年
 * @retval 1 閏年
 * @retval 0 閏年ではない
 *
 * <b> 被参照自作関数</b><br>
 * ファイルcalendar_main.c内  関数month_days<br>
 * <br>
 *
 * <b> 参照自作関数</b><br>
 * なし<br>
 * <br>
 *
 * <b> 備考</b><br>
 *  閏年の定義<br>
 * (1) 400で割り切れる年<br>
 * (2) 4で割れてかつ100で割り切れない年<br>
 * このどちらかであれば閏年である<br>
 * <br>
 * 
 */
int is_leap(int year)
{
	/* ここから処理コードの記述 */
	if(year % 4 == 0 && year % 100 != 0 || year % 400 == 0){     /* 閏年だったら */
		return 1;
	}else{                                                       /* 閏年でない */
		return 0;
	}
}

/*!
 * <b>機能</b><br>
 * year年month月の月日数を取得する<br>
 * <br>
 * 
 * @param year 年
 * @param month 月
 * @retval 月日数
 * <br>
 *
 * <b> 被参照自作関数 </b><br>
 * ファイルoutput_calendar.c内 関数month_print_calendar<br>
 * ファイルoutput_calendar.c内 関数month_print_calendar_today<br>
 * ファイルoutput_calendar_gengoh.c内　関数month_print_calendar_h1_1<br>
 * ファイルcalendar_main.c内 関数make_three_calendar<br>
 * ファイルcalendar_main.c内 関数make_three_calendar_1989<br>
 * <br>
 *
 * <b> 参照自作関数 </b><br>
 * ファイルcalendar_main.c内 関数is_leap<br>
 * <br>
 *
 * <b> 備考 </b><br>
 * 2月のみ閏年は29日、平年は28日<br>
 * <br>
 * 
 */
int month_days(int year, int month)
{
	/* ここから変数の宣言 */
	int mdays[2][13] = {                                           /* 各月の日数を設定する */
		{0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31},
		{0, 31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31}
	};
	
	/* ここから処理コードの記述 */
	if(month != 2){                             /* 2月以外ならば */
		return mdays[0][month];                 /* month月の月日数を返す */
	}else{                                      /* 2月の場合 */
		return mdays[is_leap(year)][month];     /* 閏年なら29、それ以外なら28を返す(ファイルcalendar_main.c) */
	}
} 

/*!
 * <b> 機能 </b><br>
 * year年month月day日の曜日をツェラーの公式で算出して求める<br><br>
 * 
 * @param year 年
 * @param month 月
 * @param day 日
 * 
 * @retval 曜日
 * <br>
 *
 * <b> 被参照自作関数</b><br>
 * ファイルoutput_calendar.c内 関数month_print_calendar<br>
 * ファイルoutput_calendar.c内 関数month_print_calendar_today<br>
 * ファイルoutput_calendar_gengoh.c内  関数month_print_calendar_s64_1<br>
 * ファイルoutput_calendar_gengoh.c内　関数month_print_calendar_h1_1<br>
 * ファイルcalendar_main.c内 関数make_calendar<br>
 * ファイルcalendar_main.c内 関数make_three_calendar_1989<br>
 * ファイルinsert_holidays.c内 関数Jan_holiday<br>
 * ファイルinsert_holidays.c内 関数Jan_holiday<br>
 * ファイルinsert_holidays.c内 関数Feb_holiday<br>
 * ファイルinsert_holidays.c内 関数Mar_holiday<br>
 * ファイルinsert_holidays.c内 関数Api_holiday<br>
 * ファイルinsert_holidays.c内 関数May_holiday<br>
 * ファイルinsert_holidays.c内 関数Jul_holiday<br>
 * ファイルinsert_holidays.c内 関数Sep_holiday<br>
 * ファイルinsert_holidays.c内 関数Oct_holiday<br>
 * ファイルinsert_holidays.c内 関数Nov_holiday<br>
 * ファイルinsert_holidays.c内 関数Dec_holiday<br>
 * <br>
 * 
 * <b> 参照自作関数</b><br>
 * なし
 * <br>
 *
 */
int day_of_week(int year, int month, int day)
{
	/* ここから処理コードの記述(ツェラーの公式) */
	if(month == 1 || month == 2){   
		year--;          
		month += 12;
	}
	
	return ((year + year / 4 - year / 100 + year / 400 + (13 * month + 8) / 5 + day) % 7);
}


/*!
 * <b> 機能 </b><br>
 * year年month月からの3ヶ月カレンダーを3次元配列配列に代入する<br>
 * <br>
 *
 * <b> 処理詳細</b><br>
 * (1) 初日の曜日を取得する<br>
 * (2) 月日数を取得する<br>
 * (3) 3ヶ月分のカレンダーを入れる<br>
 * <br>
 * 
 * @param year[] 年
 * @param month[] 月
 * @param cal_buf [][][] カレンダーバッファ
 * @param *week_row 週の行数
 *
 * <b> 被参照自作関数</b><br>
 * ファイルopreate_calendar.c内  関数input_ym_three<br>
 * ファイルopreate_calendar.c内  関数input_ym_three_today<br>
 * ファイルopreate_calendar_gengoh.c内  関数input_ym_three_gengoh<br>
 * ファイルopreate_calendar_gengoh.c内  関数input_ym_three_today_gengoh<br>
 * ファイルopreate_calendar_gengoh.c内　関数input_1989_three<br>
 * <br>
 *
 * <b> 参照自作関数</b><br>
 * ファイルcalendar_main.c内 関数day_of_week<br>
 * ファイルcalendar_main.c内 関数month_days<br>
 * ファイルcalendar_main.c内 関数reset<br>
 * <br>
 *
 */
void make_three_calendar(int year[COL_CAL], int month[COL_CAL], int cal_buf[COL_CAL][WEEK_ROW][WEEK_COL], int *week_row)
{
	/* ここから変数の宣言 */
	int i, j, k;                                         /* ループ変数 */
	int week_val = day_of_week(year[0], month[0], 1);    /* year年month月1日の曜日を取得する(ファイルcalendar_main.c) */
	int m_days = month_days(year[0], month[0]);          /* year年month月の日数を取得する(ファイルcalendar_main.c) */
	int day = 1;                                         /* 月始め */
	int week_max = 0;                                    /* 行数最大 */
	
    /* ここから処理コードの記述 */
	reset(cal_buf);                                     /* バッファを初期化(ファイルcalendar_main.c) */
	for(i = 0; i < COL_CAL;){								/* 3ヶ月分 */
		for(j = 0; j < WEEK_ROW;){							/* 週は最大で6周 */
			cal_buf[i][j][week_val] = day;					/* 日付を代入 */
			if(++week_val == 7){								/* 土曜なら */
				week_val = 0;									/* 日曜に設定　*/
				if(m_days > day){								/* 月末でない */
					j++;										/* 週の数を増やす */
				}
			}
			day++;
			if(m_days < day){									/* 月末まで */
				i++;											/* 次の月 */
				day = 1;										/* 初日に設定 */
				m_days = month_days(year[i], month[i]);	/* 月日数取得(ファイルcalendar_main.c) */
				if(*week_row < j + 1){
					*week_row = j + 1;
				}
				break;
			}
		}
	}
}

/*!
 * <b> 機能 </b><br>
 * 1989年1月からの3ヶ月カレンダーを3次元配列配列に代入する<br>
 * <br>
 *
 * <b> 処理詳細</b><br>
 * (1) 初日の曜日を取得する<br>
 * (2) 月日数を取得する<br>
 * (3) 3ヶ月分のカレンダーを入れる<br>
 * <br>
 * 
 * @param year[] 年
 * @param month[] 月
 * @param cal_buf [][][] カレンダーバッファ
 * @param *week_row 週の行数
 *
 * <b> 被参照自作関数</b><br>
 * ファイルopreate_calendar_gengoh.c内 関数input_1989_1to3<br>
 * <br>
 *
 * <b> 参照自作関数</b><br>
 * ファイルcalendar_main.c内 関数day_of_week<br>
 * ファイルcalendar_main.c内 関数month_days<br>
 * ファイルcalendar_main.c内 関数reset<br>
 * <br>
 * 
 */
void make_three_calendar_1989(int year[COL_CAL], int month[COL_CAL], int cal_buf[COL_CAL][WEEK_ROW][WEEK_COL], int *week_row)
{
	/* ここから変数の宣言 */
	int i, j, k, l;                                      /* ループ変数 */
	int week_val = day_of_week(year[0], month[0], 8);    /* year年month月1日の曜日を取得する(ファイルcalendar_main.c) */
	int m_days = month_days(year[0], month[0]);          /* year年month月の日数を取得する(ファイルcalendar_main.c) */
	int day = 8;                                         /* 月始め */
	int week_max = 0;                                    /* 行数最大 */
	
    /* ここから処理コードの記述 */
	l = 1;
	reset(cal_buf);                                     /* バッファを初期化(ファイルcalendar_main.c) */
	for(i = 0; i < COL_CAL;){								/* 3ヶ月分 */
		for(j = l; j < WEEK_ROW;){							/* 週は最大で6周 */
			cal_buf[i][j][week_val] = day;					/* 日付を代入 */
			if(++week_val == 7){								/* 土曜なら */
				week_val = 0;									/* 日曜に設定　*/
				if(m_days > day){								/* 月末でない */
					j++;										/* 週の数を増やす */
				}
			}
			day++;
			if(m_days < day){									/* 月末まで */
				i++;											/* 次の月 */
				day = 1;										/* 初日に設定 */
				m_days = month_days(year[i], month[i]);	/* 月日数取得(ファイルcalendar_main.c) */
				
				if(*week_row < j + 1){
					*week_row = j + 1;
				}
				l = 0;
				break;
			}
		}
	}
}

/*!
 * <b> 機能 </b><br>
 * year年month月の祝日名と祝日日付を代入する<br>
 * <br>
 * 
 * @param year 年
 * @param month 月
 * @param holiday_name[][] 祝日表示バッファ
 * @param *max_holidays 3ヶ月間の最大祝日日数
 * @param holiday_num[][] 日付代入変数
 *
 * <b> 被参照自作関数</b><br>
 * ファイルopreate_calendar.c内  関数input_ym_three<br>
 * ファイルopreate_calendar.c内  関数input_ym_three_today<br>
 * ファイルopreate_calendar_gengoh.c内  関数input_ym_three_gengoh<br>
 * ファイルopreate_calendar_gengoh.c内  関数input_ym_three_today_gengoh<br>
 * ファイルopreate_calendar_gengoh.c内　関数input_1989_three<br>
 * <br>
 *
 * <b> 参照自作関数</b><br>
 * ファイルinsert_holidays.c内 関数judge_month<br>
 * <br>
 * 
 */
void make_holiday(int year, int month, char holiday_name[HOLY_MAX][HOLY_NAME], int holiday_num[HOLY_MAX], int *max_holidays)
{
    /* ここから変数の宣言 */
	char hol_temp[HOLY_NAME];						/* 祝日用のテンポラリ配列 */
	char mon_holiday_name[HOLY_MAX][HOLY_NAME];	/* 月ごとの祝日を入れる */
	int m_holidays;									/* 月別の祝日を入れる */
	int i;
    
    /* 処理コードの記述 */
	for(i = 0; i < HOLY_MAX; i++){
		holiday_num[i] = 0;
	}
	m_holidays = judge_month(year, month, mon_holiday_name, holiday_num);	/* ファイルinsert_holiday.c */
    
	for(i = 0; i < HOLY_MAX; i++){												/* 祝日用のバッファをクリア */
		holiday_name[i][0] = '\0';
	}
	
	for(i = 0; i < m_holidays; i++){											/* その月の祝日を入れる */
		sprintf(hol_temp, "%3s", mon_holiday_name[i]);						/* 祝日名を移動させて */
		strcat(holiday_name[i], hol_temp);										/* 祝日配列に代入する */
	}
	
	for(i = m_holidays; i < HOLY_MAX; i++){									/* 空白を入れる */
		sprintf(holiday_name[i], "%21s", "");
	}
	
	if(*max_holidays < m_holidays){
		*max_holidays = m_holidays;												/* 最大祝日日数を更新する */
	}
}

/*!
 * <b> 機能 </b><br>
 * 1989年1,2,3月の祝日名と祝日日付を代入する<br>
 * <br>
 * 
 * @param year 年
 * @param month 月
 * @param holiday_name[][] 祝日表示バッファ
 * @param *max_holidays 3ヶ月間の最大祝日日数
 * @param holiday_num[][] 日付代入変数
 *
 * <b> 被参照自作関数</b><br>
 * ファイルopreate_calendar_gengoh.c内 関数input_1989_1to3<br>
 * <br>
 *
 * <b> 参照自作関数</b><br>
 * ファイルinsert_holidays.c内 関数judge_month_1989<br>
 * <br>
 * 
 */
void make_holiday_1989(int year, int month, char holiday_name[HOLY_MAX][HOLY_NAME], int holiday_num[HOLY_MAX], int *max_holidays)
{
    /* ここから変数の宣言 */
	char hol_temp[HOLY_NAME];						/* 祝日用のテンポラリ配列 */
	char mon_holiday_name[HOLY_MAX][HOLY_NAME];	/* 月ごとの祝日を入れる */
	int m_holidays;									/* 月別の祝日を入れる */
	int i;
    
    /* 処理コードの記述 */
	for(i = 0; i < HOLY_MAX; i++){
		holiday_num[i] = 0;
	}
    m_holidays = judge_month_1989(year, month, mon_holiday_name, holiday_num);	/* ファイルinsert_holidays.c */
    
	for(i = 0; i < HOLY_MAX; i++){														/* 祝日用のバッファをクリア */
		holiday_name[i][0] = '\0';
	}
	
	for(i = 0; i < m_holidays; i++){													/* その月の祝日を入れる */
		sprintf(hol_temp, "%3s", mon_holiday_name[i]);								/* 祝日名を移動させて */
		strcat(holiday_name[i], hol_temp);												/* 祝日配列に代入する */
	}
	
	for(i = m_holidays; i < HOLY_MAX; i++){											/* 空白を入れる */
		sprintf(holiday_name[i], "%21s", "");
	}
	
	if(*max_holidays < m_holidays){
		*max_holidays = m_holidays;														/* 最大祝日日数を更新する */
	}
}

/*!
 * <b> 機能 </b><br>
 * 配列を初期化する<br>
 * <br>
 *
 * @param cal_buf [][] カレンダーバッファ
 *
 * <b> 被参照自作関数</b><br>
 * ファイルcalendar_main.c内 関数make_three_calendar<br>
 * ファイルcalendar_main.c内 関数make_three_calendar_1989<br>
 * <br>
 *
 * <b> 参照自作関数</b><br>
 * なし<br>
 * <br>
 * 
 */
void reset(int cal_buf[COL_CAL][WEEK_ROW][WEEK_COL])
{
	/* ここから変数の宣言 */
	int i, j, k;
    
	/* ここから処理コードの記述 */
	for(i = 0; i < COL_CAL; i++){       
		for(j = 0; j < WEEK_ROW; j++){
			for(k = 0; k < WEEK_COL; k++)
			cal_buf[i][j][k] = 0;
		}
	}
}
