/*!
 * @file
 * @brief エラーメッセージ表示プログラム
 * @author y_okamon
 * @version 1.0(2007.04.25)　初版 
 * @version 1.1(2008.10.09)　関数use_mes内、機能追加に伴うメッセージ追加
 * @version 2.0(2011.08.15)　全ての関数、コマンド名変更によるメッセージ変更、機能追加
 */
 
/* ここからヘッダファイルの宣言 */
#include "number_set.h"
#include<stdio.h>

/*!
 * <b>機能</b><br>
 * 使用可能な月の数値を表示する<br><br>
 *
 * <b> 被参照自作関数 </b><br>
 * ファイルghcal.c内  関数month_check<br>
 * <br>
 *
 * <b> 参照自作関数 </b><br>
 * なし<br>
 * <br>
 *
 */
void month_mes()
{
	printf("ghcal : 不正な月 : %d-%dを使用してください。\n", MONTH_MIN, MONTH_MAX);
}

/*!
 * <b>機能</b><br>
 * 使用可能な年の数値を表示する<br><br>
 *
 * <b> 被参照自作関数 </b><br>
 * ファイルghcal.c内  関数year_check<br>
 * <br>
 *
 * <b> 参照自作関数 </b><br>
 * なし<br>
 * <br>
 *
 */
void year_mes()
{
	printf("ghcal : 不正な年 : %d-%dを使用してください。\n", YEAR_MIN, YEAR_MAX);
}


/*!
 * <b> 機能 </b><br>
 * 昭和の使用可能な年の数値を表示する<br><br>
 *
 * <b> 被参照自作関数 </b><br>
 * ファイルopreate_calendar_gengoh.c内 関数year_month_gengoh_check<br>
 * <br>
 *
 * <b> 参照自作関数 </b><br>
 * なし<br>
 * <br>
 *
 */
void syowa_year_mes()
{
	printf("ghcal : 不正な昭和元号 : s%d-s%dを使用してください。\n", SYOWA_MIN, SYOWA_MAX);
}

/*!
 * <b> 機能 </b><br>
 * 昭和の使用可能な年度を表示する<br>
 * <br>
 *
 * <b> 被参照自作関数 </b><br>
 * ファイルopreate_calendar_gengoh.c内 関数year_fiscal_check<br>
 * <br>
 *
 * <b> 参照自作関数 </b><br>
 * なし<br>
 * <br> 
 */
void syowa_fiscal_mes()
{
	printf("ghcal : 不正な昭和年度 : s%d-s%dを使用してください。\n", SYOWA_MIN, SYOWA_MAX -1);
}

/*!
 * <b> 機能 </b><br>
 * 平成の使用可能な年の数値を表示する<br><br>
 *
 * <b> 被参照自作関数 </b><br>
 * ファイルopreate_calendar_gengoh.c内 関数year_month_gengoh_check<br>
 * <br>
 *
 * <b> 参照自作関数 </b><br>
 * なし<br>
 * <br>
 *
 */
void heisei_year_mes()
{
	printf("ghcal : 不正な平成元号 : h%d-h%dを使用してください。\n", HEISEI_MIN, HEISEI_MAX);
}

/*!
 * <b> 機能 </b><br>
 * 平成の使用可能な年の数値を表示する<br><br>
 *
 * <b> 被参照自作関数 </b><br>
 * ファイルopreate_calendar_gengoh.c内 関数year_fiscal_check<br>
 * <br>
 *
 * <b> 参照自作関数 </b><br>
 * なし<br>
 * <br>
 *
 */
void heisei_fiscal_mes()
{
	printf("ghcal : 不正な平成元号 : h%d-h%dを使用してください。\n", HEISEI_MIN, HEISEI_MAX - 1);
}



/*!
 * <b>機能</b><br>
 * 正しいコマンドの利用方法を表示する<br><br>
 *
 * <b> 被参照自作関数 </b><br>
 * ファイルghcal.c内  関数argcheck<br>
 * <br>
 *
 * <b> 参照自作関数 </b><br>
 * なし<br>
 * <br>
 *
 */
void use_mes()
{
	printf("usage : ghcal [-3fgymnb][month] year\n");
	printf("説明 : 祝日入りのカレンダーを表示します。\n");
	printf("ghcal : 今月のカレンダーを表示\n");
	printf("ghcal -3 : 先月、今月、来月の3ヶ月分のカレンダーを表示\n");
	printf("ghcal -3 month year : year年month - 1月、year年month月, year年month + 1月の3ヶ月分のカレンダーを表示\n");
	printf("ghcal -g : 今月のカレンダーを元号で表示\n");
	printf("ghcal -g [-bnfym] : 指定されたコマンドのカレンダーを元号で表示\n");
	printf("ghcal -m : 今月のカレンダーを表示\n");
	printf("ghcal -nm : 翌月のカレンダーを表示\n");
	printf("ghcal -bm: 前月のカレンダーを表示\n");
	printf("ghcal -y : 今年のカレンダーを表示\n");
	printf("ghcal -ny: 翌年のカレンダーを表示\n");
	printf("ghcal -by: 前年のカレンダーを表示\n");
	printf("ghcal -f : 今年度のカレンダーを表示\n");
	printf("ghcal -nf: 次年度のカレンダーを表示\n");
	printf("ghcal -bf: 前年度のカレンダーを表示\n");
	printf("ghcal -f year : year年度のカレンダーを表示\n");
	printf("ghcal year : year年のカレンダーを表示\n");
	printf("ghcal month year : year年month月のカレンダーを表示\n");
	printf("month : %d-%d\n", MONTH_MIN, MONTH_MAX);
	printf("year -> 西暦入力/元号入力どちらでも可能\n");
	printf("year(西暦) : %d-%d\n",YEAR_MIN, YEAR_MAX);
	printf("year(昭和) : s%d-s%d\n",SYOWA_MIN, SYOWA_MAX);
	printf("year(平成) : h%d-h%d\n",HEISEI_MIN, HEISEI_MAX); 
}
