/*!
 * @mainpage 祝日表示カレンダーコマンドドキュメントページ
 */

/*!
 * @file
 * @brief 祝日表示カレンダーコマンドメインページ
 * 
 * @author y_okamon
 * @version 1.0(2007.04.25) 初版
 * @version 1.1(2008.10.14) <br>
 * コマンドプログラムのオプション追加について<br>
 * (1) 次月を表示する関数next_month_cal、前月を表示する関数before_month_calの参照を行う<br>
 * (2) 次年を表示する関数next_year_cal、前年を表示する関数before_year_calの参照を行う<br>
 * (3) 次年度を表示する関数next_year_cal、前年度を表示する関数before_fiscal_calの参照を行う<br>
 * <br>
 * ファイルoutput_calendar.cに参照される仮引数なしの関数について<br>
 * (1) これらの関数をCalfunc型のポインタの配列として宣言<br>
 * (2) 宣言された配列の要素番号に対応したコマンドオプションを配列として宣言<br>
 * (3) コマンドオプションを関数selectsに設定、入力されたコマンドオプションがあるかどうか調べる<br>
 * (4) 対応するコマンドオプションがあった場合は、それに対応する関数を参照するように変更<br>
 * (5) これにより、関数argcheckの行数の膨大を抑えることに成功<br>
 * <br>
 * 年月チェックについて<br>
 * (1) 関数year_month_check_main、year_check_mainを新に設計<br>
 * (2) (1)で設計された各関数の処理に関数year_checkと関数month_checkを参照させる<br>
 * (3) これにより、関数argcheckの行数の膨大を抑えることに成功<br>
 * <br>
 * その他の仕様変更<br>
 * jhcal month yearと入力されたときの参照関数を関数input_ym_calから、関数month_print_calendarに変更<br>　
 */

/* ここからヘッダファイルの宣言 */
#include<stdio.h>
#include<stdlib.h>
#include<string.h>
#include<time.h>
#include "number_set.h"

/* ここから関数のプロトタイプ宣言 */
/* ファイル ghcal.cから */
void argcheck(int argc, char *argv[]); 
void year_month_check_main(char chmonth[10], char chyear[10], int *month, int *year);  /* Ver1.1 */
void year_check_main(char chyear[10], int *year);  /* Ver1.1 */
int selects(char comarg[11]);

/* ファイルerror_mes.cからの関数 */
void month_mes();
void year_mes();
void use_mes();

/* ファイルoutput_calendar.cからの関数 */
void month_print_calendar(int year, int month);

/* ファイルopreate_calendar.cからの関数 */
void this_month_cal();       
void this_year_cal();
void this_three_month(); 
void input_ym_three(int year, int month);
void this_fiscal_cal(); 
void input_fiscal_cal(int year);
void next_month_cal();    			/* Ver1.1 */
void before_month_cal();  			/* Ver1.1 */
void next_year_cal();     			/* Ver1.1 */
void before_year_cal();   			/* Ver1.1 */
void next_fiscal_cal();   			/* Ver1.1 */
void before_fiscal_cal(); 			/* Ver1.1 */

/* ファイルopreate_calendar_gengoh.cからの関数 */
void this_month_cal_gengoh();												/* Ver2.0 */
void next_month_cal_gengoh();												/* Ver2.0 */
void before_month_cal_gengoh();												/* Ver2.0 */
void this_year_cal_gengoh();												/* Ver2.0 */
void next_year_cal_gengoh();												/* Ver2.0 */
void before_year_cal_gengoh();												/* Ver2.0 */
void this_fiscal_cal_gengoh();												/* Ver2.0 */
void next_fiscal_cal_gengoh();												/* Ver2.0 */
void before_fiscal_cal_gengoh();											/* Ver2.0 */
void opreate_year_month_gengoh(char chmonth[10], char chyear[10]);		/* Ver2.0 */
void opreate_year_gengoh(char chyear[10]);								/* Ver2.0 */
void opreate_fiscal_gengoh(char chyear[10]);								/* Ver2.0 */

/* ファイルargcheck.cからの関数 */
void year_check(int year);
void month_check(int month);

/*!
 * <b> 機能</b><br>
 * 関数をポインタ配列で管理する<br>
 * <br>
 */
typedef void (*Calfunc)(void);

/*!
 * <b> 機能 </b><br>
 * 入力されたコマンドのタイプによってさまざまなタイプのカレンダーを表示する。<br><br>
 *
 * @param argc 入力された引数の数
 * @param *argv[] 入力された引数の内容
 * @retval 0 正常終了
 *
 * <b> 被参照自作関数 </b><br>
 * なし<br>
 * <br>
 *
 * <b> 参照自作関数</b><br>
 * ファイルghcal.c内  関数argcheck<br>
 * <br>
 *
 */
 
int main(int argc, char *argv[])
{
	/* ここから処理コード */
    argcheck(argc, argv);         /* 引数チェックを行う */
    return 0;
}
/*!
 * <b> 機能</b><br>
 * 入力された引数をチェックして、様々なカレンダーを表示する<br><br>
 * 
 * @param argc 入力された引数の数
 * @param *argv[] 入力された引数の内容<br>
 * 
 * <b>処理詳細</b><br>
 * ファイルerror_mes.c内　関数use_mes()の内容を参照
 *
 * <b> 被参照自作関数 </b><br>
 * ファイルghcal.c内  関数main<br>
 * <br>
 *
 * <b> 参照自作関数 </b><br>
 * ファイルopreate_calendar.c内  関数this_month_cal<br>
 * ファイルopreate_calendar.c内  関数month_print_calendar<br>
 * ファイルopreate_calendar.c内  関数this_year_cal<br>
 * ファイルopreate_calendar.c内  関数input_year_cal<br>
 * ファイルopreate_calendar.c内  関数this_fiscal_cal<br>
 * ファイルopreate_calendar.c内  関数input_fiscal_cal<br>
 * ファイルopreate_calendar.c内  関数this_three_month<br>
 * ファイルopreate_calendar.c内  関数input_three_month<br>
 * ファイルopreate_calendar.c内  関数next_month_cal<br>
 * ファイルopreate_calendar.c内  関数before_month_cal<br>
 * ファイルopreate_calendar.c内  関数next_year_cal<br>
 * ファイルopreate_calendar.c内  関数before_year_cal<br>
 * ファイルopreate_calendar.c内  関数next_fiscal_cal<br>
 * ファイルopreate_calendar.c内  関数before_year_cal<br>
 * ファイルopreate_calendar_gengoh.c内  関数this_month_cal_gengoh<br>
 * ファイルopreate_calendar_gengoh.c内　関数next_month_cal_gengoh<br>
 * ファイルopreate_calendar_gengoh.c内  関数before_month_cal_gengoh<br>
 * ファイルopreate_calendar_gengoh.c内　関数opreate_year_month_gengoh<br>
 * ファイルopreate_calendar_gengoh.c内  関数opreate_year_gengoh<br>
 * ファイルopreate_calendar_gengoh.c内  関数this_fiscal_cal_gengoh<br>
 * ファイルopreate_calendar_gengoh.c内  関数next_fiscal_cal_gengoh<br>
 * ファイルopreate_calendar_gengoh.c内  関数before_fiscal_cal_gengoh<br>
 * ファイルopreate_calendar_gengoh.c内  関数input_fiscal_cal_gengoh<br>
 * ファイルopreate_calendar_gengoh.c内  関数opreate_fiscal_gengoh<br>
 * ファイルoutput_calendar.c内  関数month_print_calendar<br>
 * ファイルghcal.c内  関数year_check_main<br>
 * ファイルghcal.c内  関数year_month_check_main<br>
 * ファイルghcal.c内  関数selects<br>
 * ファイルerror.c内  関数use_mes<br>
 *<br>
 */
void argcheck(int argc, char *argv[])
{
 	/* ここから変数の宣言 */
 	int month;                         /* 引数入力された月を代入する */
 	int year;                          /* 引数入力された年を代入する */
    /* 関数ポインタ配列の宣言 */
	/* ファイルopreate_calendar.cから */
	Calfunc calendar[] = {this_month_cal, next_month_cal, before_month_cal, this_year_cal, next_year_cal, before_year_cal,
this_fiscal_cal, next_fiscal_cal, before_fiscal_cal, this_three_month, this_month_cal_gengoh};
	/* ファイルopreate_calendar_gengoh.cから */
	Calfunc calendar_gengoh[] = {this_month_cal_gengoh, next_month_cal_gengoh, before_month_cal_gengoh, this_year_cal_gengoh,
next_year_cal_gengoh, before_year_cal_gengoh, this_fiscal_cal_gengoh, next_fiscal_cal_gengoh, before_fiscal_cal_gengoh};
	int array_num = sizeof(calendar) / sizeof(calendar[0]);  /* 配列の要素数を取得 */
	int snum;                                                /* 引数の選択番号 */ 
    
 	/* ここから処理コードの記述 */
 	if(argc == 1){																/* ghcalのみの入力だった場合 */
		this_month_cal();															/* 今月のカレンダーを表示する(ファイルopreate_calendar.c) */
	}else if(argc == 2){															/* ghcal xxxxと入力した場合 */
		snum = selects(argv[1]); 												/* コマンドに対応する番号を取得する(ファイルghcal.c) */
		if(snum != array_num){							 						/* 選択番号が要素数と一致しない */
			calendar[snum](); 													/* 関数を呼び出す */
		}else{                                                      		/* ghcal yearと入力された場合 */
			if(strlen(argv[1]) == 4){											/* yearの部分が西暦表示(ファイルghcal.c) */
				year_check_main(argv[1], &year);								/* コマンドラインからの入力が有効かどうか調べる(Ver1.1 ファイルghcal.c) */
 				input_year_cal(year);											/* year年のカレンダーを表示する(ファイルopreate_calendar.c) */        
			}else if((strlen(argv[1]) == 2) || (strlen(argv[1]) == 3)){	/* ghcal sXX or ghcal hYYと入力された(Ver2.0) */
				opreate_year_gengoh(argv[1]);									/* 元号に対応した年間カレンダーを表示する(ファイルopreate_calendar_gengoh.c) */			
			}else{
				use_mes();															/* 使い方を表示する(ファイルerror_mes.c) */
 				exit(1);															/* プログラムを終了する */		
			}		
		}
 	}else if(argc == 3){															/* ghcal xxxx yyyy と入力された場合 */
		if(strcmp(argv[1], "-f") == 0){ 										/* ghcal -f yearと入力されたら */
			if(strlen(argv[2]) == 4){											/* 西暦表示された場合 */
				year_check_main(argv[2], &year); 								/* コマンドラインからの入力が有効かどうか調べる(Ver1.1 ファイルghcal.c) */
 				input_fiscal_cal(year);											/* year年度のカレンダーを表示する(ファイルopreate_calendar.c) */
			}else if((strlen(argv[2]) == 2) || (strlen(argv[2]) == 3)){		/* 元号形式で入力された */
				opreate_fiscal_gengoh(argv[2]);									/* 元号に対応した年度カレンダーを表示する(ファイルopreate_calendar_gengoh.c) */
			}else{
				use_mes();															/* 使い方を表示する(ファイルerror_mes.c) */
 				exit(1);															/* プログラムを終了する */	
			}         			
		}else if(strcmp(argv[1], "-g") == 0){									/* ghcal -g xxxxと入力された場合(Ver2.0) */
			snum = selects(argv[2]);												/* xxxxに対応する番号を取得する(ファイルghcal.c) */
			if(snum < array_num -1){												/* 選択番号が要素数と一致しない */
				calendar_gengoh[snum]();											/* 関数を呼び出す */
			}
 		}else{ 																	/* ghcal month yearが入力されたら */	
			if(strlen(argv[2]) == 4){											/* yearの部分が西暦表示だったら */
				year_month_check_main(argv[1], argv[2], &month, &year);		/* コマンドラインからの入力が有効かどうか調べる(ファイルghcal.c) */
				printf("      %d年%2d月\n", year, month);						/* 指定された年月のカレンダーを表示 */
        		month_print_calendar(year, month); 							/* year年month月のカレンダーを表示する(ファイルoutput_calendar.c) */
 			}else if(strlen(argv[2]) == 2 || strlen(argv[2]) == 3){		/* yearが元号入力 */
				opreate_year_month_gengoh(argv[1], argv[2]);					/* ファイルopreate_calendar_gengoh.c */
			}
		}
 	}else if(argc == 4){															/* ghcal -3 month yearと入力されたら */
		year_month_check_main(argv[2], argv[3], &month, &year); 			/* コマンドラインからの入力が有効かどうか調べる(ファイルghcal.c) */
 		input_ym_three(year, month);											/* 3ヶ月カレンダーを表示する(ファイルopreate_calendar.c) */	
	}else{																			/* 間違った入力の場合 */
 		use_mes();																	/* 使い方を表示する */
 		exit(1);																	/* プログラムを終了する */
 	}	
}
 
/*!
 * <b>機能</b><br>
 * (1) コマンドラインからの入力された数字を数値に変換<br>
 * (2) 変換された数値が有効な年月かどうか調べる<br>
 * <br>
 * 
 * @param chmonth 月数字
 * @param chyear 年数字
 * @param *month 月
 * @param *year 年
 *
 * <b> 被参照自作関数 </b><br>
 * ファイルghcal.c内  関数argcheck<br>
 * <br>
 *
 * <b> 参照自作関数 </b><br>
 * ファイルghcal.c内  関数month_check<br>
 * ファイルghcal.c内  関数year_check<br>
 * <br>
 * 
 */
void year_month_check_main(char chmonth[10], char chyear[10], int *month, int *year)
{
    /* ここから処理コードの記述 */
	*month = (int)atoi(chmonth);  /* 入力された引数を数値に変換する */
 	*year = (int)atoi(chyear);    /* 入力された引数を数値として変換する */
 	month_check(*month);          /* 月の有効範囲を調べる(ファイルghcal.c) */
	year_check(*year);            /* 年の有効範囲を調べる(ファイルghcal.c) */
}

/*!
 * <b>機能</b><br>
 * (1) コマンドラインからの入力された数字を数値に変換<br>
 * (2) 変換された数値が有効な年かどうか調べる<br>
 * <br>
 * 
 * @param chyear[] 年数字
 * @param *year 年
 *
 * <b> 被参照自作関数</b><br>
 * ファイルghcal.c内  関数argcheck<br>
 * <br>
 *
 * <b> 参照自作関数</b><br>
 * ファイルghcal.c内  関数year_check<br>
 * <br>
 *
 */
void year_check_main(char chyear[10], int *year)
{
    /* ここから処理コードの記述 */
	*year = (int)atoi(chyear);	/* 入力された引数を数値として変換する */
	year_check(*year);			/* 年の有効範囲を調べる(ファイルghcal.c) */
}
 
 
/*!
 * <b>機能</b><br>
 * 入力された年の有効範囲をチェックする<br>
 * <br>
 * 
 * @param year 年
 *
 * <b> 被参照自作関数</b><br>
 * ファイルghcal.c内  関数year_month_check_main<br>
 * ファイルghcal.c内  関数year_check_main<br>
 * <br>
 *
 * <b> 参照自作関数</b><br>
 * ファイルerror_mes.c内  関数year_mes<br>
 * <br>  
 *  
 */
void year_check(int year)
{
	/* ここから処理コードの記述 */
	if(year < YEAR_MIN || year > YEAR_MAX){	/* 1949〜2150以外の数値が入力されたら */
		year_mes(); 								/* 正しい利用方法を表示する。(ファイルerror_mes.c)*/
		exit(1);									/* プログラムを終了する */
	}
}

/*!
 * <b>機能</b><br>
 * 入力された月の有効範囲をチェックする
 * 
 * @param month 月
 *
 * <b> 被参照自作関数</b><br>
 * ファイルghcal.c内  関数year_month_check_main<br>
 * <br>
 *
 * <b> 参照自作関数</b><br>
 * ファイルerror_mes.c内  関数month_mes<br>
 * ファイルopreate_calendar_gengoh.c内　関数year_month_gengoh_check<br>
 * <br>  
 * 
 */
void month_check(int month)
{
	/* ここから処理コードの記述 */
	if(month < MONTH_MIN || month > MONTH_MAX){	/* 1〜12以外の数値が入力された場合 */
		month_mes(); 									/* 正しい利用法を表示する (ファイルerror_mes.c)*/
		exit(1);  										/* プログラムを終了する */
	}		
}

/*!
 * <b> 機能 </b><br>
 * 入力コマンドがあるかどうか調べる <br><br>
 *
 * @param comarg[] コマンドラインからの入力
 * @retval i 配列の番号 
 *
 * <b> 被参照自作関数</b><br>
 * ファイルghcal.c内 関数argcheck<br>
 * <br>
 * 
 * <b> 参照自作関数</b><br>
 * なし<br>
 * <br>
 *
 */
int selects(char comarg[11])
{
    /* ここから変数の宣言 */
	int i = 0; 																									/* ループ変数 */
	char command[][11] = {"-m", "-nm", "-bm", "-y", "-ny", "-by", "-f", "-nf", "-bf", "-3", "-g"};		/* コマンド */  	
	int array_num = sizeof(command) / sizeof(command[0]);  													/* 要素数 */
    
    /* ここから処理コードの記述 */
	while(i != array_num){						/* 該当するコマンドがなくなるまで */
		if(strcmp(comarg, command[i]) == 0){	/* コマンドがあった*/
			break;									/* ループブレイク */
		}else{										/* コマンドがなかった */
			i++;									/* 次の要素へ */
		}
	}
	return i;										/* 要素番号を返す */
}

