/*!
 * @file 
 * @brief 祝日を代入するプログラム
 * @author y_okamon
 * @version 1.0(2007.04.25) 初版
 * @version 1.1(2008.10.09) <br>
 * 祝日にも色を付けるための仕様変更<br>
 * (1)各関数、日付を入れる整数型配列を仮引数に設定<br>
 * (2)関数XXX_holiday内の処理、祝日名、祝日日付、日数更新処理について<br>
 * (2.1) 関数input_holidayを新に設定し、祝日名と祝日日付と日数の更新を行う<br>
 * (2.2) (2.1)から関数XXX_holidayの行数を削減<br>
 * @version 2.0(2011.06.29)<br>
 * 1948年から施行された祝日法に基づき、1949年から2150年までの祝日に対応<br>
 */

/* ここからヘッダファイルの宣言 */
#include<stdio.h>
#include<stdlib.h>
#include<string.h>
#include "holiday_lists.h"
#include "number_set.h"

/* ここから関数のプロトタイプ宣言 */
void Jan_holiday(int year, char holiday_name[HOLY_MAX][HOLY_NAME], int *m_holidays, int holiday_num[HOLY_MAX]);
void Jan_holiday_1989(int year, char holiday_name[HOLY_MAX][HOLY_NAME], int *m_holidays, int holiday_num[HOLY_MAX]);
void Feb_holiday(int year, char holiday_name[HOLY_MAX][HOLY_NAME], int *m_holidays, int holiday_num[HOLY_MAX]);
void Mar_holiday(int year, char holiday_name[HOLY_MAX][HOLY_NAME], int *m_holidays, int holiday_num[HOLY_MAX]);
void Api_holiday(int year, char holiday_name[HOLY_MAX][HOLY_NAME], int *m_holidays, int holiday_num[HOLY_MAX]);
void May_holiday(int year, char holiday_name[HOLY_MAX][HOLY_NAME], int *m_holidays, int holiday_num[HOLY_MAX]);
void Jul_holiday(int year, char holiday_name[HOLY_MAX][HOLY_NAME], int *m_holidays, int holiday_num[HOLY_MAX]);
void Sep_holiday(int year, char holiday_name[HOLY_MAX][HOLY_NAME], int *m_holidays, int holiday_num[HOLY_MAX]);
void Oct_holiday(int year, char holiday_name[HOLY_MAX][HOLY_NAME], int *m_holidays, int holiday_num[HOLY_MAX]);
void Nov_holiday(int year, char holiday_name[HOLY_MAX][HOLY_NAME], int *m_holidays, int holiday_num[HOLY_MAX]);
void Dec_holiday(int year, char holiday_name[HOLY_MAX][HOLY_NAME], int *m_holidays, int holiday_num[HOLY_MAX]);
void input_holiday(char holiday_name[HOLY_MAX][HOLY_NAME], int *m_holidays, int holiday_num[HOLY_MAX], char holiday_list[HOLY_MAX], int days, int week_val);
int year_check_vapoint(int year);  /* ver2.0 */

/*!
 * <b>機能</b><br>
 * 入力された月による祝日有無を判定する<br>
 * <br>
 * 
 * @param year 年
 * @param month 月
 * @param holiday_name[][] 祝日名代入する配列
 * @param holiday_num[][] 祝日日付代入配列(Ver1.1)
 * @retval m_holidays 月ごとの祝日日数を返す
 * <br> 
 *
 * <b> 被参照自作関数</b><br>
 * ファイルcalendar_main.c内 関数make_calendar<br>
 * ファイルoutput_calendar.c内  関数month_print_calendar<br>
 * ファイルoutput_calendar.c内  関数month_print_calendar_today<br>
 * <br>
 *
 * <b> 参照自作関数</b><br>
 * ファイルinsert_holidays.c内 関数Jan_holiday<br>
 * ファイルinsert_holidays.c内 関数Feb_holiday<br>
 * ファイルinsert_holidays.c内 関数Mar_holiday<br>
 * ファイルinsert_holidays.c内 関数Api_holiday<br>
 * ファイルinsert_holidays.c内 関数May_holiday<br>
 * ファイルinsert_holidays.c内 関数Jul_holiday<br>
 * ファイルinsert_holidays.c内 関数Sep_holiday<br>
 * ファイルinsert_holidays.c内 関数Oct_holiday<br>
 * ファイルinsert_holidays.c内 関数Nov_holiday<br>
 * ファイルinsert_holidays.c内 関数Dec_holiday<br>
 * <br>
 * 
 */
int judge_month(int year, int month, char holiday_name[HOLY_MAX][HOLY_NAME], int holiday_num[HOLY_MAX])
{
	/* ここから変数の宣言 */
	int m_holidays = 0;                    /* 1ヶ月の祝日日数 */
	
	/* ここから処理コードの詳細 */
	switch(month){																/* 月別の祝日名を代入する */
		case 1 :																	/* 1月ならば */
			Jan_holiday(year, holiday_name, &m_holidays, holiday_num); 		/* 1月の祝日を入れる(ファイルinsert_holidays.c) */
			break;
		case 2 :																	/* 2月ならば */
			if(year >= 1967)														/* 1967年以降 */			
				Feb_holiday(year, holiday_name, &m_holidays, holiday_num);	/* 2月の祝日を入れる(ファイルinsert_holidays.c) */
			break;
		case 3 :                                                        	/* 3月ならば */
			Mar_holiday(year, holiday_name, &m_holidays, holiday_num);  	/* 3月の祝日を入れる(ファイルinsert_holidays.c) */
			break;
		case 4 :																	/* 4月ならば */
			Api_holiday(year, holiday_name, &m_holidays, holiday_num);		/* 4月の祝日を入れる(ファイルinsert_holidays.c) */
			break;
		case 5 :																	/* 5月ならば */
			May_holiday(year, holiday_name, &m_holidays, holiday_num);		/* 5月の祝日を入れる(ファイルinsert_holidays.c) */
			break;
		case 6 :																	/* 6月ならばそのまま */
			break; 
		case 7 :																	/* 7月ならば */
			if(year >= 1996)														/* 1996年以降 */
				Jul_holiday(year, holiday_name, &m_holidays, holiday_num);	/* 7月の祝日を入れる(ファイルinsert_holidays.c) */
			break;
		case 8 :																	/* 8月ならばそのまま */
			break;
		case 9 : 																	/* 9月ならば */
			Sep_holiday(year, holiday_name, &m_holidays, holiday_num);		/* 9月の祝日を入れる(ファイルinsert_holidays.c) */          
			break;
		case 10 :																	/* 10月ならば */
			if(year >= 1966)														/* 1966年以降 */			
				Oct_holiday(year, holiday_name, &m_holidays, holiday_num);	/* 10月の祝日を入れる(ファイルinsert_holidays.c) */
			break;
		case 11 : 																	/* 11月ならば */
			Nov_holiday(year, holiday_name, &m_holidays, holiday_num);		/* 11月の祝日を入れる(ファイルinsert_holidays.c) */
			break;
		case 12 :																	/* 12月ならば */
			if(year >= 1989)														/* 1989年以降 */	
				Dec_holiday(year, holiday_name, &m_holidays, holiday_num);	/* 12月の祝日を入れる(ファイルinsert_holidays.c) */
			break;
	}
	return m_holidays; 															/* 祝日の日数を返す */	      
}

/*!
 * <b>機能</b><br>
 * 入力された月による祝日有無を判定する(平成元年1月～3月)<br>
 * <br>
 * 
 * @param year 年
 * @param month 月
 * @param holiday_name[][] 祝日名代入する配列
 * @param holiday_num[][] 祝日日付代入配列(Ver1.1)
 * @retval m_holidays 月ごとの祝日日数を返す
 * <br> 
 *
 * <b> 被参照自作関数</b><br>
 * ファイルcalendar_main.c内 関数make_calendar_1989<br>
 * <br>
 *
 * <b> 参照自作関数</b><br>
 * ファイルinsert_holidays.c内 関数Jan_holiday_1989<br>
 * ファイルinsert_holidays.c内 関数Feb_holiday<br>
 * ファイルinsert_holidays.c内 関数Mar_holiday<br>
 */
int judge_month_1989(int year, int month, char holiday_name[HOLY_MAX][HOLY_NAME], int holiday_num[HOLY_MAX])
{
	/* ここから変数の宣言 */
	int m_holidays = 0;                    /* 1ヶ月の祝日日数 */
	
	/* ここから処理コードの詳細 */
	switch(month){																	/* 月別の祝日名を代入する */
		case 1 :																		/* 1月ならば */
			Jan_holiday_1989(year, holiday_name, &m_holidays, holiday_num);	/* 1月の祝日を入れる(ファイルinsert_holidays.c) */
			break;
		case 2 :																		/* 2月ならば */		
			Feb_holiday(year, holiday_name, &m_holidays, holiday_num);			/* 2月の祝日を入れる(ファイルinsert_holidays.c) */
			break;
		case 3 :																		/* 3月ならば */
			Mar_holiday(year, holiday_name, &m_holidays, holiday_num);			/* 3月の祝日を入れる(ファイルinsert_holidays.c) */
			break;
	}
	return m_holidays;																/* 祝日の日数を返す */	      
}


/*!
 * <b>機能</b><br>
 * 元旦(1月1日)と成人の日(1月第2月曜日)を配列に代入する<br>
 * 元旦が日曜の場合は2日が振替休日となる<br>
 * <br>
 * 
 * @param year 年
 * @param holiday_name[][] 祝日名入用配列
 * @param *m_holidays 月単位の祝日日数
 * @param holiday_num[] 日付代入用配列(Ver1.1)
 *
 * <b> 被参照自作関数</b><br>
 * ファイルinsert_holidays.c内 関数judge_month<br>
 * <br>
 *
 * <b> 参照自作関数</b><br>
 * ファイルcalendar_main.c内 関数day_of_week<br>
 * ファイルinsert_holidays.c内 関数input_holiday<br>
 * <br>
 * 
 */
void Jan_holiday(int year, char holiday_name[HOLY_MAX][HOLY_NAME], int *m_holidays, int holiday_num[HOLY_MAX])
{
	/* ここから変数の宣言 */
	int week_val = day_of_week(year, 1, 1);	/* 1月1日の曜日を算出する(15日も同じ曜日)(ファイルcalendar_main.c) */
  
	/* ここから処理コードの記述 */
	input_holiday(holiday_name, m_holidays, holiday_num, New_Year, 1, week_val);						/* 1月1日は元旦となる(ファイルinsert_holidays.c) */
	if(week_val == 0 && year >= 1973){ 																	/* 1月1日が日曜日かつ1973年以降だったら */
		input_holiday(holiday_name, m_holidays, holiday_num, Compensatory, 2, 1);						/* 1月2日は振替休日となる(ファイルinsert_holidays.c) */
	}
	if(year >= 2000){																							/* 2000年以降 */
		input_holiday(holiday_name, m_holidays, holiday_num, Coming_Age, sec_mon[week_val], 1);		/* 1月の第2月曜日は成人の日となる(ファイルinsert_holidays.c) */ 
	}else if(year <= 1999){																					/* 1999年以前 */
		input_holiday(holiday_name, m_holidays, holiday_num, Coming_Age, 15, week_val); 				/* 1月15日は成人の日となる(ファイルinsert_holidays.c) */ 
		if(week_val == 0 && year >= 1973){																	/* 1月15日が日曜日かつ1973年以降なら */
			input_holiday(holiday_name, m_holidays, holiday_num, Compensatory, 16, 1);				/* 1月2日は振替休日となる(ファイルinsert_holidays.c) */
		}
	}
}

/*!
 * <b>機能</b><br>
 * 1989年のカレンダーを<br>
 * 元旦が日曜の場合は2日が振替休日となる<br>
 * <br>
 * 
 * @param year 年
 * @param holiday_name[][] 祝日名入用配列
 * @param *m_holidays 月単位の祝日日数
 * @param holiday_num[] 日付代入用配列(Ver1.1)
 *
 * <b> 被参照自作関数</b><br>
 * ファイルinsert_holidays.c内 関数judge_month_1989<br>
 * <br>
 *
 * <b> 参照自作関数</b><br>
 * ファイルinsert_holidays.c内 関数input_holiday<br>
 * <br>
 * 
 */
void Jan_holiday_1989(int year, char holiday_name[HOLY_MAX][HOLY_NAME], int *m_holidays, int holiday_num[HOLY_MAX])
{
	/* ここから変数の宣言 */
	int week_val = day_of_week(year, 1, 15);	/* 1月15日の曜日を算出する(ファイルcalendar_main.c) */
  
	/* ここから処理コードの記述 */
	input_holiday(holiday_name, m_holidays, holiday_num, Coming_Age, 15, week_val);		/* 1月15日は成人の日となる(ファイルinsert_holidays.c) */ 
	input_holiday(holiday_name, m_holidays, holiday_num, Compensatory, 16, 1);			/* 1月16日は振替休日となる(ファイルinsert_holidays.c) */
}

/*!
 * <b>関数名</b><br>
 * 2月祝日関数<br><br>
 * 
 * <b>機能</b><br>
 * 建国記念日(2月11日)を祝日代入配列に代入する<br>
 * 建国記念日が日曜のときは12日が振替休日となる<br>
 * <br>
 * 
 * @param year 年
 * @param holiday_name[][] 祝日名入用配列
 * @param *m_holidays 月単位の祝日日数
 * @param holiday_num[] 日付代入用配列(Ver1.1)
 *
 * <b> 被参照自作関数</b><br>
 * ファイルinsert_holidays.c内 関数judge_month<br>
 * <br>
 *
 * <b> 参照自作関数</b><br>
 * ファイルinsert_holidays.c内 関数input_holiday<br>
 * ファイルcalendar_main.c内 関数day_of_week<br>
 * <br>
 * 
 */
void Feb_holiday(int year, char holiday_name[HOLY_MAX][HOLY_NAME], int *m_holidays, int holiday_num[HOLY_MAX])
{
	/* ここから変数の宣言 */
	int week_val = day_of_week(year, 2, 11);  /* 2月11日の曜日を算出する(ファイルcalendar_main.c) */
	
	/* ここから処理コードの記述 */         
	input_holiday(holiday_name, m_holidays, holiday_num, National_Fundation, 11, week_val);		/* 2月11日は建国記念日となる(ファイルinsert_holidays.c) */
	if(week_val == 0 && year >= 1973){																	/* 2月11日が日曜日かつ1973年以降ならば */
		input_holiday(holiday_name, m_holidays, holiday_num, Compensatory, 12, 1);				/* 2月12日は振替休日となる(ファイルinsert_holidays.c) */
	} 
}

/*!
 * <b> 機能 </b><br>
 * 春分日(3月20日前後)を祝日代入関数に代入する<br>
 * 春分日が日曜のときは翌日が振替休日となる<br>
 * <br>
 * 
 * @param year 年
 * @param holiday_name[][] 祝日名代入配列
 * @param *m_holidays 月単位の祝日日数
 * @param holiday_num[] 日付代入用配列(Ver1.1)
 *
 * <b> 被参照自作関数</b><br>
 * ファイルinsert_holidays.c内 関数judge_month<br>
 * ファイルinsert_holidays.c内 関数judge_month_1989<br>
 * ファイルcalendar_main.c内 関数day_of_week<br>
 * <br>
 *
 * <b> 参照自作関数</b><br>
 * ファイルinsert_holidays.c内 関数vernal_autumnal<br>
 * ファイルcalendar_main.c内 関数day_of_week<br>
 * ファイルinsert_holidays.c内 関数year_check_vapoint<br>
 * <br>
 * 
 */
void Mar_holiday(int year, char holiday_name[HOLY_MAX][HOLY_NAME], int *m_holidays, int holiday_num[HOLY_MAX])
{
	/* ここから変数の宣言 */
	double vernal_points[3] = {20.8357, 20.8431, 21.8150};	/* 春分点通過点(Ver2.0) */
	int year_array[3] = {1983, 1980, 1980};					/* 春分点を求めるのに利用すべき数値 */
	int use_point = year_check_vapoint(year);					/* 春分点を求めるときに利用すべき番号(ファイルinsert_holidays.c) */               
	int ver_days;													/* 春分日 */	
	int week_val;													/* 曜日の値を算出する */

	/* ここから処理コードの記述 */
	ver_days = vernal_autumnal(vernal_points[use_point], year_array[use_point], year);				/* 春分日を算出する(ファイルinsert_holidays.c) */
	week_val = day_of_week(year, 3, ver_days);															/* 春分日の曜日を算出する(ファイルcalendar_main.c) */ 
	input_holiday(holiday_name, m_holidays, holiday_num, Vernal_Equinox, ver_days , week_val);		/* 春分日の日にちを代入する(ファイルinsert_holidays.c) */
	if(week_val == 0 && year >= 1973){																		/* 春分日が日曜日かつ1973年以降なら */
		input_holiday(holiday_name, m_holidays, holiday_num, Compensatory, ver_days + 1, 1);		/* 春分日の翌日は振替休日(ファイルinsert_holidays.c) */
	}	
}

/*!
 * <b> 機能</b><br>
 * 4月29日の祝日名(天皇誕生日(1988年まで)、みどりの日(1989年から2006年まで), 昭和の日(2007年以降))を配列に代入する<br>
 * 29日が日曜なら30日が振替休日となる<br>
 * <br>
 * 
 * @param year 年
 * @param holiday_name[][] 祝日名代入用配列
 * @param *m_holidays 月単位の祝日
 * @param holiday_num[] 日付代入用配列(Ver1.1)
 *
 * <b> 被参照自作関数</b><br>
 * ファイルinsert_holidays.c内 関数judge_month<br>
 * <br>
 *
 * <b> 参照自作関数</b><br>
 * ファイルcalendar_main.c内 関数day_of_week<br>
 * ファイルinsert_holidays.c内 関数input_holiday<br>
 * <br>
 *
 */
void Api_holiday(int year, char holiday_name[HOLY_MAX][HOLY_NAME], int *m_holidays, int holiday_num[HOLY_MAX])
{
	/* ここから変数の宣言 */
	int week_val = day_of_week(year, 4, 29);    /* 4月29日の曜日を算出する(ファイルcalendar_main.c) */  
	
	/* ここから処理コードの記述 */	
	if(year <= 1988){																							/* 1988年までは */
		input_holiday(holiday_name, m_holidays, holiday_num, The_Emperor_Birthday, 29, week_val);	/* 4月29日は天皇誕生日(ファイルinsert_holidays.c) */
	}else if(year >= 1989 && year <= 2006){																/* 1989年から2006年までは */
		input_holiday(holiday_name, m_holidays, holiday_num, Greeey, 29, week_val);					/* 4月29日はみどりの日(ファイルinsert_holidays.c) */
	}else if(year >= 2007){																					/* 2007年以降 */
		input_holiday(holiday_name, m_holidays, holiday_num, Showa, 29, week_val);					/* 4月29日は昭和の日(ファイルinsert_holiday.c) */
	}

	if(week_val == 0 && year >= 1973){																		/* 4月29日が日曜日かつ1973年以降だったら */
		input_holiday(holiday_name, m_holidays, holiday_num, Compensatory, 30, 1);					/* 4月30日は振替休日となる(ファイルinsert_holiday.c) */
	}

}

/*!
 * <b> 機能</b><br>
 * 憲法記念日(5月3日),みどりの日(2007年以降の5月4日), こどもの日(5月5日)を祝日代入配列に代入する<br>
 * こどもの日(2006年まで)/三日間のうちのどちらかが日曜の場合、6日が振替休日となる<br>
 * <br>
 * 
 * @param year 年
 * @param holiday_name[][] 祝日名代入関数
 * @param *m_holidays 月単位の祝日日数
 * @param holiday_num[] 日付代入配列(Ver1.1)
 *
 * <b> 被参照自作関数</b><br>
 * ファイルinsert_holidays.c内 関数judge_month<br>
 * <br>
 *
 * <b> 参照自作関数</b><br>
 * ファイルcalendar_main.c内 関数day_of_week<br>
 * ファイルinsert_holidays.c内 関数input_holiday<br>
 * <br>
 * 
 */
void May_holiday(int year, char holiday_name[HOLY_MAX][HOLY_NAME], int *m_holidays, int holiday_num[HOLY_MAX])
{
	/* ここから変数の宣言 */
	int week_val[4];		/* 曜日値 */
	int i;					/* ループ変数 */
	
	/* ここから処理コードの記述 */
	for(i = 0; i < 4; i++){
		week_val[i] = day_of_week(year, 5, i + 3);																/* 5月3日〜5月5日の曜日を算出する(ファイルcalendar_main.c) */
	}
	
	input_holiday(holiday_name, m_holidays, holiday_num, Constitation, 3, week_val[0]);					/* 5月3日は憲法記念日(ファイルinsert_holidays.c) */
	if(year < 2007){																									/* 2006年までは */
		if(week_val[1] == 1 && year >= 1973){																		/* 5月4日が月曜日かつ1973年以降ならば */
			input_holiday(holiday_name, m_holidays, holiday_num, Compensatory, 4, 1);						/* 振替休日(ファイルinsert_holidays.c) */            
		}else if(week_val[1] > 1 && year >= 1988){																/* 5月4日が火〜土のときで1988年以降なら */
			input_holiday(holiday_name, m_holidays, holiday_num, National_Holiday, 4, week_val[1]);		/* 5月4日は国民の休日(ファイルinsert_holidays.c)  */            
		}
		input_holiday(holiday_name, m_holidays, holiday_num, Children, 5, week_val[2]);					/* 5月5日はこどもの日(ファイルinsert_holidays.c) */
		if(week_val[3] == 1 && year >= 1973){																		/* 5月6日が月曜日 */
			input_holiday(holiday_name, m_holidays, holiday_num, Compensatory, 6, 1);						/* 5月6日は振替休日(ファイルinsert_holidays.c) */
		}                                                                                        
	}else{																												/* 2007年以降 */
		input_holiday(holiday_name, m_holidays, holiday_num, Greeey, 4, week_val[1]);						/* 5月4日はみどりの日(ファイルinsert_holidays.c) */
		input_holiday(holiday_name, m_holidays, holiday_num, Children, 5, week_val[2]);					/* 5月5日はこどもの日(ファイルinsert_holidays.c) */ 
		if(week_val[0] == 0 || week_val[1] == 0 || week_val[2] == 0){                                	/* 5月3日〜5月5日までのどちらかが日曜日である */
			input_holiday(holiday_name, m_holidays, holiday_num, Compensatory, 6, week_val[3]);			/* 5月6日は振替休日(ファイルinsert_holidays.c) */ 
		}
	}
}

/*!
 * <b> 機能</b><br>
 * 海の日を代入する<br>
 * 2002年までは海の日が日曜なら21日が振替休日となる<br>
 * <br>
 * 
 * @param year 年
 * @param holiday_name[][] 祝日名代入配列
 * @param *m_holidays 月単位の祝日日数
 * @param holiday_num[] 日付代入配列(Ver1.1)
 *
 * <b> 被参照自作関数</b><br>
 * ファイルinsert_holidays.c内 関数judge_month<br>
 * <br>
 *
 * <b> 参照自作関数</b><br>
 * ファイルinsert_holidays.c内 関数input_holiday<br>
 * ファイルcalendar_main.c内 関数day_of_week<br>
 * <br>
 * 
 */
void Jul_holiday(int year, char holiday_name[HOLY_MAX][HOLY_NAME], int *m_holidays, int holiday_num[HOLY_MAX])
{
	/* ここから変数の宣言 */
	int week_val;		/* 曜日値 */
	
	/* ここから処理コードの記述 */
	if(year < 2003){																						/* 2002年までは */
		week_val = day_of_week(year, 7, 20);															/* 7月20日の曜日を算出する(ファイルcalendar_main.c) */
		input_holiday(holiday_name, m_holidays, holiday_num, Ocean, 20, week_val);				/* 7月20日は海の日(ファイルinsert_holidays.c) */ 
		if(week_val == 0){																				/* 7月20日が日曜日である */
			input_holiday(holiday_name, m_holidays, holiday_num, Compensatory, 21, 1);			/* 7月21日は振替休日(ファイルinsert_holidays.c) */  
		}
	}else{																									/* 2003年以降 */
		week_val = day_of_week(year, 7, 1);															/* 7月1日の曜日を算出して(ファイルcalendar_main.c) */
		input_holiday(holiday_name, m_holidays, holiday_num, Ocean, thi_mon[week_val], 1);		/* 7月第3月曜は海の日(ファイルinsert_holidays.c) */   
	}
}

/*!
 * <b> 機能</b><br>
 * 敬老の日と秋分の日を配列に代入する<br>
 * 敬老の日(2002年まで)、秋分の日が日曜なら翌日は振替休日となる<br>
 * <br>
 * 
 * @param year
 * @param holiday_name[][] 祝日名代入用配列
 * @param *m_holidays 月単位の祝日日数
 * @param holiday_num[] 日付代入配列(Ver1.1)
 *
 * <b> 被参照自作関数</b><br>
 * ファイルinsert_holidays.c内 関数judge_month<br>
 * <br>
 *
 * <b> 参照自作関数</b><br>
 * ファイルinsert_holidays.c内 関数vernal_autumnal<br>
 * ファイルinsert_holidays.c内 関数input_holiday<br>
 * ファイルinsert_holidays.c内 関数year_check_vapoint<br>
 * ファイルcalendar_main.c内 関数day_of_week<br>
 * <br>
 *
 */
void Sep_holiday(int year, char holiday_name[HOLY_MAX][HOLY_NAME], int *m_holidays, int holiday_num[HOLY_MAX])
{
	/* ここから変数の宣言 */
	int week_val_r = day_of_week(year, 9, 1);				/* 9月1日の曜日を取得する(9月15日と同じ曜日 ファイルcalendar_main.c) */
	double autumnal[3] = {23.2588, 23.2488, 24.2488};	/* 秋分点通過点(Ver2.0) */
	int year_array[3] = {1983, 1980, 1980};				/* 計算に利用する年 */  	
	int use_point = year_check_vapoint(year);				/* 秋分点を求めるときに利用すべき番号(ファイルinsert_holidays.c) */
	int au_days;     											/* 秋分日 */
	int week_val_au; 											/* 秋分日の曜日 */ 

	/* ここから処理コードの記述 */
	au_days = vernal_autumnal(autumnal[use_point], year_array[use_point], year);									/* 秋分日を算出する(ファイルinsert_holidays.c) */
	week_val_au = day_of_week(year, 9, au_days);																			/* 秋分日の曜日を算出(ファイルday_of_week.c) */
	if(year >= 1966 && year <= 2002){																						/* 1966年から2002年までは */
		input_holiday(holiday_name, m_holidays, holiday_num, Respect_for_the_Aged, 15, week_val_r);				/* 9月15日が敬老の日(ファイルinsert_holidays.c) */ 
		if(week_val_r == 0 && year >= 1973){																				/* 9月15日が日曜日かつ1973年以降であれば */
			input_holiday(holiday_name, m_holidays, holiday_num, Compensatory, 16, 1);								/* 16日は振替休日(ファイルinsert_holidays.c) */  
		}
	}else if(year >= 2003){																									/* 2003年からは */
		input_holiday(holiday_name, m_holidays, holiday_num, Respect_for_the_Aged, thi_mon[week_val_r], 1);		/* 9月第3月曜は敬老の日(ファイルinsert_holidays.c) */  
		if((au_days - thi_mon[week_val_r]) == 2){																			/* 敬老の日の明後日が秋分の日の場合 */
			input_holiday(holiday_name, m_holidays, holiday_num, National_Holiday, au_days - 1, 2);				/* 間の火曜日が国民の祝日となる(ファイルinsert_holidays.c) */         
		}
	}
	input_holiday(holiday_name, m_holidays, holiday_num, Autumnal_Equinox, au_days, week_val_au);				/* 秋分日(ファイルinsert_holidays.c) */ 
	if(week_val_au == 0 && year >= 1973){																					/* 秋分の日が日曜日かつ1973年以降だったら */
		input_holiday(holiday_name, m_holidays, holiday_num, Compensatory, au_days + 1, 1); 						/* 秋分日の翌日が振替休日 */
	}   
}

/*!
 * <b> 機能</b><br>
 * 体育の日を配列に代入する<br><br>
 * 
 * @param year 年
 * @param holiday_name[][] 祝日名代入配列
 * @param *m_holidays 月単位の祝日日数
 * @param holiday_num[] 祝日日付代入配列(Ver1.1)
 *
 * <b> 被参照自作関数</b><br>
 * ファイルinsert_holidays.c内 関数judge_month<br>
 * <br>
 *
 * <b> 参照自作関数</b><br>
 * ファイルcalendar_main.c内 関数day_of_week<br>
 * ファイルinsert_holidays.c内 関数input_holiday<br>
 * <br>
 *
 */
void Oct_holiday(int year, char holiday_name[HOLY_MAX][HOLY_NAME], int *m_holidays, int holiday_num[HOLY_MAX])
{
	/* ここから変数の宣言 */
	int week_val;		/* 曜日取得 */
    
    /* ここから処理コードの記述 */
	if(year >= 2000){																										/* 2000年以降 */
		week_val = day_of_week(year, 10, 1);																			/* 10月1日の曜日を取得する(ファイルcalendar_main.c) */
		input_holiday(holiday_name, m_holidays, holiday_num, Health_and_Sports, sec_mon[week_val], 1);		/* 10月第2月曜は体育の日(ファイルinsert_holidays.c) */
	}else{																													/* 1999年以前 */
		week_val = day_of_week(year, 10, 10);																			/* 10月10日の曜日を取得する(ファイルcalendar_main.c)*/
		input_holiday(holiday_name, m_holidays, holiday_num, Health_and_Sports, 10, week_val);				/* 10月10日は体育の日(ファイルinsert_holidays.c) */
		if(week_val == 0 && year >= 1973){																				/* 10月10日が日曜日かつ1973年以降なら */
			input_holiday(holiday_name, m_holidays, holiday_num, Compensatory, 11, 1);							/* 10月11日は振替休日(ファイルinsert_holidays.c) */
		}	
	}
}

/*!
 * <b> 機能 </b><br>
 * 文化の日(11月3日)と勤労感謝の日(11月23日)を配列に代入する<br>
 * 文化の日、勤労感謝の日が日曜の場合は翌日は振替休日となる<br>
 * <br>
 * 
 * @param year 年
 * @param holiday_name[][] 祝日名代入用配列
 * @param *m_holidays 月単位の祝日日数
 * @param holiday_num[] 日付代入配列(Ver1.1)
 *
 * <b> 被参照自作関数</b><br>
 * ファイルinsert_holidays.c内 関数judge_month<br>
 * <br>
 *
 * <b> 参照自作関数</b><br>
 * ファイルinsert_holidays.c内 関数input_holiday<br>
 * ファイルcalendar_main.c内 関数day_of_week<br>
 * <br>
 * 
 */
void Nov_holiday(int year, char holiday_name[HOLY_MAX][HOLY_NAME], int *m_holidays, int holiday_num[HOLY_MAX])
{
	/* ここから変数の宣言 */
	int week_val_c = day_of_week(year, 11, 3);		/* 11月3日の曜日を取得(ファイルinsert_holidays.c) */
	int week_val_l = day_of_week(year, 11, 23);		/* 11月23日の曜日を取得(ファイルinsert_holidays.c) */ 
    
    /* ここから処理コードの記述 */
	input_holiday(holiday_name, m_holidays, holiday_num, Culture, 3, week_val_c);					/* 11月3日は文化の日(ファイルinsert_holidays.c) */ 
	if(week_val_c == 0 && year >= 1973){     																/* 11月3日が日曜日なら */
		input_holiday(holiday_name, m_holidays, holiday_num, Compensatory, 4, 1);					/* 11月4日は振替休日となる(ファイルinsert_holidays.c) */
	}    
	input_holiday(holiday_name, m_holidays, holiday_num, Labor_Thanksgiving, 23, week_val_l);		/* 11月23日は勤労感謝の日(ファイルinsert_holidays.c) */
	if(week_val_l == 0 && year >= 1973){																	/* 11月23日が日曜日なら */
		input_holiday(holiday_name, m_holidays, holiday_num, Compensatory, 24, 1);					/* 11月24日は振替休日(ファイルinsert_holidays.c) */
	}
}

/*!
 * <b> 機能</b><br>
 * 天皇誕生日(12月23日)を配列に代入する<br>
 * 天皇誕生日が日曜の場合は24日が振替休日となる<br>
 * <br>
 * 
 * @param year 年
 * @param holiday_name[][] 祝日名代入用関数
 * @param *m_holidays 月単位の祝日日数
 * @param holiday_num[] 日付代入配列(Ver1.1)
 *
 * <b> 被参照自作関数</b><br>
 * ファイルinsert_holidays.c内 関数judge_month<br>
 * <br>
 *
 * <b> 参照自作関数</b><br>
 * ファイルinsert_holidays.c内　関数input_holiday<br>
 * ファイルcalendar_main.c内 関数day_of_week<br>
 * <br>
 * 
 */
void Dec_holiday(int year, char holiday_name[HOLY_MAX][HOLY_NAME], int *m_holidays, int holiday_num[HOLY_MAX])
{
	/* ここから変数の宣言 */
	int week_val = day_of_week(year, 12, 23);		/* 12月23日の曜日を算出する(ファイルcalendar_main.c) */
    
    /* ここから処理コードの記述 */
	input_holiday(holiday_name, m_holidays, holiday_num, The_Emperor_Birthday, 23, week_val);		/* 12月23日は天皇誕生日(ファイルinsert_holidays.c) */
	if(week_val == 0){																						/* 12月23日が日曜日 */
		input_holiday(holiday_name, m_holidays, holiday_num, Compensatory, 24, 1);					/* 12月24日は振替休日(ファイルinsert_holidays.c) */
	}
}

/*!
 * <b> 機能</b><br>
 * 祝日を入れる<br>
 * <br>
 * 
 * @param holiday_name[][] 祝日代入用関数
 * @param *m_holidays 月単位の祝日日数
 * @param holiday_num[] 日付代入配列
 * @param holiday_list[] ヘッダファイルにある祝日名
 * @param days 日付
 * @param week_val 曜日値
 *
 * <b> 被参照自作関数</b><br>
 * ファイルinsert_holidays.c内 関数Jan_holiday<br>
 * ファイルinsert_holidays.c内 関数Jan_holiday_1989<br>
 * ファイルinsert_holidays.c内 関数Feb_holiday<br>
 * ファイルinsert_holidays.c内 関数Mar_holiday<br>
 * ファイルinsert_holidays.c内 関数Api_holiday<br>
 * ファイルinsert_holidays.c内 関数May_holiday<br>
 * ファイルinsert_holidays.c内 関数Jul_holiday<br>
 * ファイルinsert_holidays.c内 関数Sep_holiday<br>
 * ファイルinsert_holidays.c内 関数Oct_holiday<br>
 * ファイルinsert_holidays.c内 関数Nov_holiday<br>
 * ファイルinsert_holidays.c内 関数Dec_holiday<br>
 * <br>
 *
 * <b> 参照自作関数</b><br>
 * なし<br>
 * <br>
 * 
 */
void input_holiday(char holiday_name[HOLY_MAX][HOLY_NAME], int *m_holidays, int holiday_num[HOLY_MAX], char holiday_list[HOLY_MAX], int days, int week_val)
{
	/* ここから変数の宣言 */
	int index = (*m_holidays);	/* 現時点での祝日日数が配列の要素番号となる */
    
	/* ここから処理コードの記述 */
	sprintf(holiday_name[index], " %2d(%2s) %2s", days, weeklist[week_val], holiday_list);	/* 祝日名、日付を入れる */
	holiday_num[index] = days;																			/* 日付を代入 */
	(*m_holidays)++;																						/* 祝日日数を1日増やす */	
}

/*!
 * <b> 機能</b><br>
 * 春分日と秋分日を計算で算出する
 * 
 * @param points 春分日、秋分日の通過平均日
 * @param va_year 春分、秋分日の基準年
 * @param year 対象年
 * 
 * @retval va_days 春分日、秋分日
 *
 * <b> 被参照自作関数 </b><br>
 * ファイルinsert_holidays.c内 関数Mar_holiday<br>
 * ファイルinsert_holidays.c内 関数Sep_holiday<br>
 * <br>
 *
 * <b> 参照自作関数</b><br>
 * なし<br>
 * <br>
 *
 */
int vernal_autumnal(double points, int va_year, int year)
{
	/* ここから変数の宣言 */
	int va_days;			/* 春分日or秋分日 */
	double moving;		/* 1年ごとの移動日 */
	int reset;				/* 閏年によるリセット量 */
	
	/* ここから処理コードの記述 */
	moving = (double)(year - 1980) * 0.242194;	/* 1年ごとの春分秋分通過日を算出する */
	reset = (year - va_year) / 4;					/* 閏年によるリセット量を算出する */
	va_days = (int)(points + moving) - reset;		/* 春分秋分日を設定する */
	return va_days;                                    
}

/*!
 * <b> 機能 </b><br>
 * 入力された年から利用すべき春分/秋分点を決定する<br>
 * <br>
 *
 * @param year 年
 *
 * <b> 被参照自作関数 </b><br>
 * ファイルinsert_holidays.c内 関数Mar_holiday<br>
 * ファイルinsert_holidays.c内 関数Sep_holiday<br>
 * <br>
 *
 * <b> 参照自作関数 </b><br>
 * なし<br>
 * <br>
 */
int year_check_vapoint(int year)
{
	/* ここから処理コードの記述 */
	if(year <= 1979){
		return 0;
	}else if(year >= 1980 && year <= 2099){
		return 1;
	}else if(year >= 2100){
		return 2;
	}
}
