/*!
 * @file
 * @brief カレンダー操作プログラム
 * @author y_okamon
 * @version 1.1(2008.10.14)<br>
 * 新ファイル作成の意図<br>
 * ファイルoutput_calendarから、カレンダー操作部分をこのファイルに移行
 * <br>
 * コマンドオプション追加においての関数の設計<br>
 * (1) 次月を表示する関数next_month_cal、前月を表示する関数before_month_calを設計<br>
 * (2) 次年を表示する関数next_year_cal、前年を表示する関数before_year_calを設計<br>
 * (3) 次年度を表示する関数next_fiscal_cal、前年を表示する関数before_year_calを設計<br>
 * <br>
 * 日付取得用構造体DAY_DATEを設定したときの関数の仕様変更<br>
 * (1) システム上の日付を取得する関数get_todayを設計、返り値はDAY_DATE型の変数となる<br>
 * (2) (1)から、next、this、before系の関数の行数を削減することに成功<br>
 * <br>
 * 関数input_ym_calの削除について<br>
 * この関数は、各関数から年月を引数として受け取って、month_print_calendarを参照していた。
 * しかし、この関数を参照しないで、各関数がmonth_print_calendarを直接参照しても結果が変わらないと判明
 * よって、この関数は不要となったので削除<br>
 * <br>
 * カレンダーを横に3個表示するときの仕様変更<br>
 * (1) 関数input_ym_three内3次元の文字列配列から整数型配列に変更<br>
 * (2) 関数input_ym_three内3ヶ月分の年と月を入れる配列を設定<br>
 * (3) 関数make_calendarのカレンダー設定部分を関数make_three_calendarに変更<br>
 * (4) 配列に3ヶ月分のデータを入れることで、ループで関数make_holidayを処理(関数make_calendarの祝日部分を関数make_holidayに変更)<br>
 * <br>
 * 関数val_updateの削除について<br>
 * 最初はループカウンタ、最大祝日日数、最大週数の更新を関数input_ym_threeに入れると長くなってしまうために設定した。
 * しかし、最大祝日日数は関数make_holidayで、最大週数はmake_calendarで処理させるようにした
 * それにより、ループカウンタは関数input_ym_threeで処理すればよいので、関数val_updateを削除<br>
 * <br>
 * 年間および年度カレンダー表示処理<br>
 * 年間および年度間カレンダーの表示はprint_calendar_yearで行っていたが、仕様を変更した関数input_ym_threeを4回呼び出して、
 * 実験を行ってみたら同じような結果が得られた。そのため関数print_calendar_yearは不要となったので削除<br>
 * <br>
 */

/* ここからヘッダファイルの宣言 */
#include<stdio.h>
#include<string.h>
#include<time.h>
#include"number_set.h"

/* ここから関数のプロトタイプの宣言 */
/* ファイルinsert_holiday.cから */
int judge_month(int year, int month, char holiday_name[HOLY_MAX][HOLY_NAME], int holiday_days[HOLY_MAX]);

/* ファイルopreate_calendar.cから */
void month_print_calendar(int year, int month);
void month_print_calendar_today(int year, int month, int day);
void input_year_cal(int year);
void input_fiscal_cal(int year);
DAY_DATE get_today();
void row_calendar_main(int cal_buf[COL_CAL][WEEK_ROW][WEEK_COL], int week_max, int holiday_num[COL_CAL][HOLY_MAX]);
void row_calendar_main_today(int cal_buf[COL_CAL][WEEK_ROW][WEEK_COL], int week_max, int holiday_num[COL_CAL][HOLY_MAX], int month, int day, int arr_month[COL_CAL]);
void input_ym_three(int year, int month);
void input_ym_three_today(int year, int month, DAY_DATE today);
void print_calendar_three(int year[COL_CAL], int month[COL_CAL]);
void make_array(int arr_year[COL_CAL], int arr_month[COL_CAL], int year, int month);

/* ファイルoutput_calendar.cから */
void row_holiday(char holiday_name[COL_CAL][HOLY_MAX][HOLY_NAME], int hol_max);

/*!
 * <b> 機能</b><br>
 * 今月のカレンダーを表示する<br>
 * <br>
 *
 * <b> 処理詳細</b><br>
 * (1) システム上の年月を取得する。<br>
 * (2) 取得した年月を関数month_print_calendar_gengohで処理<br>
 * <br>
 *
 * <b> 被参照自作関数 </b><br>
 * ファイルghcal.c内  関数argcheck<br>
 * <br>
 *
 * <b> 参照自作関数 </b><br>
 * ファイルopreate_calendar.c内  関数month_print_calendar_today<br>
 * ファイルopreate_calendar.c内　関数get_today<br>
 * <br>
 *
 */
void this_month_cal() 
{
    /* ここから変数の宣言 */
	DAY_DATE today = get_today();	/* 現在の年月日を取得する(ファイルopreate_calendar.c) */
    
    /* ここから処理コードの記述 */
	printf("      %d年%2d月\n", today.year, today.month);					/* 年月を表示 */
	month_print_calendar_today(today.year, today.month, today.day);	/* 今月のカレンダーを表示する(ファイルoutput_calendar.c) */
}

/*!
 * <b> 機能</b><br>
 * 来月のカレンダーを表示する<br>
 * <br>
 *
 * <b> 処理詳細</b><br>
 * (1) システム上の年月を取得する。<br>
 * (2) 取得した年月を関数month_print_calendarで処理<br>
 * <br>
 *
 * <b> 被参照自作関数 </b><br>
 * ファイルghcal.c内  関数argcheck<br>
 * <br>
 *
 * <b> 参照自作関数 </b><br>
 * ファイルopreate_calendar.c内  関数month_print_calendar<br>
 * ファイルopreate_calendar.c内　関数get_today<br>
 * <br>
 */
void next_month_cal() 
{
	/* ここから変数の宣言 */
	DAY_DATE today = get_today();	/* 日付けを取得する(ファイルopreate_calendar.c) */

	/* ここから処理コードの記述 */ 
	if(today.month == MONTH_MAX){										/* 今月が12月なら */
		printf("      %d年%2d月\n", today.year + 1, MONTH_MIN);		/* 翌年の1月のカレンダーを表示 */
		month_print_calendar(today.year + 1, MONTH_MIN);				/* ファイルoutput_calendar.c */
	}else{
		printf("      %d年%2d月\n", today.year, today.month + 1);		/* 指定された年月のカレンダーを表示 */						
		month_print_calendar(today.year, today.month + 1);			/* ファイルoutput_calendar.c */
	}
}

/*!
 * <b> 機能</b><br>
 * 先月のカレンダーを表示する<br>
 * <br>
 *
 * <b> 処理詳細</b><br>
 * (1) システム上の年月を取得する。<br>
 * (2) 取得した年月を関数month_print_calendarで処理<br>
 * <br>
 *
 * <b> 被参照自作関数 </b><br>
 * ファイルghcal.c内  関数argcheck<br>
 * <br>
 *
 * <b> 参照自作関数 </b><br>
 * ファイルopreate_calendar.c内  関数month_print_calendar<br>
 * ファイルopreate_calendar.c内  関数get_today<br>
 * <br>
 *
 */
void before_month_cal() 
{
    /* ここから変数の宣言 */
	DAY_DATE today = get_today();	/* 日付を取得(ファイルopreate_calendar.c) */

    /* ここから処理コードの記述 */
	if(today.month == MONTH_MIN){										/* 今月が1月なら */
		printf("      %d年%2d月\n", today.year - 1, MONTH_MAX);		/* 前年の12月のカレンダーを表示 */
		month_print_calendar(today.year - 1, MONTH_MAX);				/* ファイルoutput_calendar.c */
	}else{																	/* それ以外 */
		printf("      %d年%2d月\n", today.year , today.month - 1);	/* 指定された年月のカレンダーを表示 */
		month_print_calendar(today.year, today.month - 1);			/* ファイルoutput_calendar.c */
    }
}

/*!
 * <b> 機能 </b><br>
 * 今年の年間カレンダーを表示する<br>
 * <br>
 *
 * <b> 処理詳細 </b><br>
 * (1) システム上の年を取得する<br>
 * (2) 取得した年をinput_year_cal関数で処理<br>
 * <br>
 * 
 * <b> 被参照自作関数</b><br>
 * ファイルghcal.c内 関数argcheck<br>
 * <br>
 *
 * <b> 参照自作関数</b><br>
 * ファイルopreate_calendar.c内  関数input_ym_three_today<br>
 * ファイルopreate_calendar.c内　関数input_ym_three<br>
 * ファイルopreate_calendar.c内  関数get_today<br>
 * <br>
 * 
 */
void this_year_cal() 
{
    /* ここから変数の宣言 */
	DAY_DATE today = get_today();	/* 日付データ構造体変数(ファイルopreate_calendar.c) */
	int i;
	int month = 0;
    
    /* ここから処理コードの記述 */
	printf("%34d年\n\n", today.year);																/* 先頭にxxxx年のカレンダーを表示させる(Ver2.0) */
	for(i = 0; i < 4; i++){
		month = month + 3;
		if(today.month == month - 2 || today.month == month - 1 || today.month == month){
			input_ym_three_today(today.year, month - 1, today);									/* ファイルopreate_calendar.c */
		}else{
			input_ym_three(today.year, month -1);													/* ファイルopreate_calendar.c */
		}
	}
}

/*!
 * <b> 機能 </b><br>
 * 翌年の年間カレンダーを表示する<br>
 * <br>
 *
 * <b> 処理詳細 </b><br>
 * (1) システム上の年を取得する<br>
 * (2) 取得した年+1をinput_year_cal関数で処理<br>
 * <br>
 * 
 * <b> 被参照自作関数</b><br>
 * ファイルghcal.c内 関数argcheck<br>
 * <br>
 *
 * <b> 参照自作関数</b><br>
 * ファイルopreate_calendar.c内  関数input_year_cal<br>
 * ファイルopreate_calendar.c内  関数get_today<br>
 * <br>
 */
void next_year_cal() 
{
	/* ここから変数の宣言 */
	DAY_DATE today = get_today();	/* 本日の日付を取得(ファイルopreate_calendar.c) */
    
	/* ここから処理コードの記述 */
	input_year_cal(today.year + 1);	/* カレンダーを表示する(ファイルopreate_calendar.c) */
}

/*!
 * <b> 機能 </b><br>
 * 昨年の年間カレンダーを表示する<br>
 * <br>
 *
 * <b> 処理詳細 </b><br>
 * (1) システム上の年を取得する<br>
 * (2) 取得した年から１を引いた年をinput_year_cal関数で処理<br>
 * <br>
 * 
 * <b> 被参照自作関数</b><br>
 * ファイルghcal.c内 関数argcheck<br>
 * <br>
 *
 * <b> 参照自作関数</b><br>
 * ファイルopreate_calendar.c内  関数input_year_cal<br>
 * ファイルopreate_calendar.c内  関数get_today<br>
 * <br>
 * 
 */
void before_year_cal() 
{
	/* ここから変数の宣言 */
	DAY_DATE today = get_today();	/* 本日の日付を取得する(ファイルopreate_calendar.c) */

	/* ここから処理コードの記述 */
	input_year_cal(today.year - 1);	/* カレンダーを表示する(ファイルopreate_calendar.c */
}


/*!
 * <b> 機能</b><br>
 * 入力された年の年間カレンダーを表示させる<br>
 * <br>
 *
 * <b> 処理詳細</b><br>
 * 横に3ヶ月分のカレンダーを表示する。<br>
 * <br>
 *
 * @param year 年
 *  
 *
 * <b> 被参照自作関数</b><br>
 * ファイルghcal.c内  関数argcheck<br>
 * ファイルopreate_calendar.c内  関数next_year_cal<br>
 * ファイルopreate_calendar.c内　関数before_year_cal<br>
 * <br>
 *
 * <b> 参照自作関数</b><br>
 * ファイルopreate_calendar.c内  関数input_ym_three<br>
 * <br>
 *
 */
void input_year_cal(int year) 
{
	/* ここから変数の宣言 */
	int month = 1;     /* 月 */
	int i;             /* ループ変数 */

	/* ここから処理コードの記述 */
	printf("%34d年\n\n", year);			/* 先頭にxxxx年のカレンダーを表示させる(Ver2.0) */
	for(i = 0; i < 4; i++){				/* 3ヶ月ごと4回 */
		month = 3 * i + 2;				/* 2,5,8,11月を基準とする */
		input_ym_three(year, month);	/* 3ヶ月カレンダーを表示(ファイルopreate_calendar.c) */
	}
}

/*!
 * <b>機能</b><br>
 * 今年度のカレンダーを表示する<br><br>
 *
 * <b> 処理詳細 </b><br>
 * (1) システム上の年と月を取得する<br>
 * (2) 取得した月が1月〜3月の間であったら、年を1年マイナス<br>
 * (3) 取得した年をinput_year_cal関数で処理<br>
 * <br>
 * 
 * <b> 被参照自作関数</b><br>
 * ファイルghcal.c内 関数argcheck<br>
 * <br>
 *
 * <b> 参照自作関数</b><br>
 * ファイルopreate_calendar.c内  関数input_ym_three_today<br>
 * ファイルopreate_calendar.c内  関数input_ym_three<br>
 * ファイルopreate_calendar.c内  関数get_today<br>
 * <br>
 * 
 */
void this_fiscal_cal() 
{
    /* ここから変数の宣言 */
	DAY_DATE today = get_today();	/* 日付の構造体(ファイルopreate_calendar.c) */
	int month = 3;
	int i;
    
    /* ここから処理コードの記述 */
	if(today.month < 4){																				/* 現在の月が1月〜3月まで(Ver1.01) */
		today.year--;                                                                       /* 年度はyear -1 年度(Ver1.01) */
	}
	printf("%34d年度\n\n", today.year); 															/* 先頭にxxxx年度と表示させる(Ver2.0) */
	for(i = 0; i < 3; i++){																			/* 4月〜12月まで */
		month = month + 3;																			/* 3ヶ月更新 */
		if(today.month == month - 2 || today.month == month - 1 || today.month == month){
			input_ym_three_today(today.year, month - 1, today);									/* ファイルopreate_calendar.c */
		}else{
			input_ym_three(today.year, month -1);													/* ファイルopreate_calendar.c */
		}
	}
	month = 3;
	if(today.month == month - 2 || today.month == month - 1 || today.month == month){
		input_ym_three_today(today.year + 1, month - 1, today);									/* ファイルopreate_calendar.c */
	}else{
		input_ym_three(today.year + 1, month -1);													/* ファイルopreate_calendar.c */
	} 
}

/*!
 * <b>機能</b><br>
 * 翌年度のカレンダーを表示する<br><br>
 *
 * <b> 処理詳細 </b><br>
 * (1) システム上の年と月を取得する<br>
 * (2) 年をプラス1する。<br>
 * (3) 取得した月が1月〜3月の間であったら、年を1年マイナス<br>
 * (4) 取得した年をinput_year_cal関数で処理<br>
 * <br>
 * 
 * <b> 被参照自作関数</b><br>
 * ファイルghcal.c内 関数argcheck<br>
 * <br>
 *
 * <b> 参照自作関数</b><br>
 * ファイルopreate_calendar.c内  関数input_fiscal_cal<br>
 * ファイルopreate_calendar.c内　関数get_today<br>
 * <br>
 * 
 */
void next_fiscal_cal() 
{
    /* ここから変数の宣言 */
	DAY_DATE today = get_today();	/* 日付を取得する(ファイルopreate_calendar.c) */

    /* ここから処理コードの記述 */
	if(today.month < 4) {				/* 現在の月が1月〜3月まで(Ver1.01) */
		today.year--;						/* 年度はyear -1 年度(Ver1.01) */
	}
	input_fiscal_cal(today.year + 1);	/* カレンダーを表示する(ファイルopreate_calendar.c) */
}


/*!
 * <b>機能</b><br>
 * 前年度のカレンダーを表示する<br><br>
 *
 * <b> 処理詳細 </b><br>
 * (1) システム上の年と月を取得する<br>
 * (2) 年をマイナス1する。
 * (3) 取得した月が1月〜3月の間であったら、年を1年マイナス<br>
 * (4) 取得した年をinput_year_cal関数で処理<br>
 * <br>
 * 
 * <b> 被参照自作関数</b><br>
 * ファイルghcal.c内 関数argcheck<br>
 * <br>
 *
 * <b> 参照自作関数</b><br>
 * ファイルopreate_calendar.c内  関数get_today<br?
 * ファイルopreate_calendar.c内  関数input_fiscal_cal<br>
 * <br>
 * 
 */
void before_fiscal_cal() 
{
    /* ここから変数の宣言 */
	DAY_DATE today = get_today();	/* 今日の日付を取得する(ファイルopreate_calendar.c) */

    /* ここから処理コードの記述 */
	if(today.month < 4) {				/* 現在の月が1月〜3月まで */
		today.year --;					/* 年度はyear -1 年度(Ver1.01) */
	}
	input_fiscal_cal(today.year - 1);	/* カレンダーを表示する(ファイルopreate_calendar.c) */
}
 
/*!
 * <b> 機能</b><br>
 * 入力された年の年度カレンダーを表示させる<br>
 * <br>
 *
 * @param year 年
 *
 * <b> 被参照自作関数</b><br>
 * ファイルghcal.c内  関数argcheck<br>
 * ファイルopreate_calendar.c内  関数next_fiscal_cal<br>
 * ファイルopreate_calendar.c内　関数before_fiscal_cal<br>
 * <br>
 *
 * <b> 参照自作関数</b><br>
 * ファイルopreate_calendar.c内  関数input_ym_three<br>
 * <br>
 * 
 */
void input_fiscal_cal(int year) 
{
    /* ここから変数の宣言 */
	int month = 1;	/* 月 */
	int i;				/* ループ変数 */    

    /* ここから処理コードの記述 */
	printf("%34d年度\n\n", year); 		/* 先頭にxxxx年度と表示させる(Ver2.0) */
	for(i = 1; i < 4; i++){ 				/* 4月〜12月まで */
		month = 3 * i + 2;				/* 中央にくる月を算出 */
		input_ym_three(year, month);	/* カレンダーを表示(ファイルopreate_calendar.c) */
	}
	input_ym_three(year + 1, 2);		/* ファイルopreate_calendar.c */
}

/*!
 * <b> 機能</b><br>
 * 前月、今月、来月の3ヶ月を表示する<br><br>
 *
   <b> 処理詳細 </b><br>
 * (1) システム上の年月を取得する<br>
 * (2) 取得した年をinput_ym_three関数で処理<br>
 * <br>
 * 
 * <b> 被参照自作関数</b><br>
 * ファイルghcal.c内 関数argcheck<br>
 * <br>
 *
 * <b> 参照自作関数</b><br>
 * ファイルopeate_calendar.c内  関数input_ym_three_today<br>
 * <br>
 * 
 */
void this_three_month() 
{
    /* ここから変数の宣言 */
	DAY_DATE today = get_today();	/* 日付用構造体変数宣言(ファイルopreate_calendar.c) */
    
    /* ここから処理コードの記述 */
	input_ym_three_today(today.year, today.month, today);	/* 前月　基準月、次月の3ヶ月分を表示(ファイルopreate_calendar.c) */
}

/*!
 * <b> 機能</b>
 * 先月、今月、来月の3ヶ月カレンダーを表示する<br><br>
 *
 * <b> 処理詳細</b>
 * (1) 入力/システムから獲得した月によって以下の処理をする。<br>
 * (1.1) 月が1月なら、前年の12月からシステム上の年の2月までのカレンダーを作る。<br>
 * (1.2) 月が12月なら、11月から翌年の1月までのカレンダーを作る。<br>
 * (1.3) それ以外の月なら、前の月から翌月までのカレンダーを作る。<br>
 * (2) 3ヶ月分のカレンダーが作成したら、横にカレンダーを表示する<br>
 * <br>
 * 
 * @param year 年
 * @param month 年
 *
 * <b> 被参照自作関数</b><br>
 * ファイルghcal.c内  関数argcheck<br>
 * ファイルoutput_calendar内  関数input_year_cal<br>
 * ファイルoutput_calendar内  関数input_fiscal_cal<br>
 * <br>
 *
 * <b> 参照自作関数</b><br>
 * ファイルopreate_calendar.c内 関数make_array<br>
 * ファイルcalendar_main.c内  関数make_three_calendar<br>
 * ファイルcalendar_main.c内　関数make_holiday<br>
 * ファイルoutput_calendar.c内  関数print_calendar_three<br>
 * ファイルoutput_calendar.c内  関数row_calendar_main<br>
 * ファイルoutput_calendar.c内  関数row_holiday<br>
 * <br>
 *
 */
void input_ym_three(int year, int month) 
{
    /* ここから変数の宣言 */
	int cal_buf[COL_CAL][WEEK_ROW][WEEK_COL];			/* カレンダーのバッファ */
	char holiday_name[COL_CAL][HOLY_MAX][HOLY_NAME];	/* 月別の祝日 */ 
	int holiday_num[COL_CAL][HOLY_MAX];				/* 祝日用日付 */
	int hol_max = 0;										/* 3ヶ月間での最大祝日日数 */
	int week_max = 4;										/* 週数 */
	int i;
	int arr_year[COL_CAL];
	int arr_month[COL_CAL];
    
    /* ここから処理コードの記述 */
	make_array(arr_year, arr_month, year, month);														/* ファイルopreate_calendar.c */
	make_three_calendar(arr_year, arr_month, cal_buf, &week_max);									/* ファイルcalendar_main.c */
	for(i = 0; i < COL_CAL; i++){
		make_holiday(arr_year[i], arr_month[i], holiday_name[i], holiday_num[i], &hol_max);		/* ファイルcalendar_main.c */
	}
	print_calendar_three(arr_year, arr_month);														/* ファイルoutput_calendar.c */
	row_calendar_main(cal_buf, week_max, holiday_num);												/* ファイルoutput_calendar.c */
	row_holiday(holiday_name, hol_max);																/* ファイルoutput_calendar.c */
}

/*!
 * <b> 機能</b>
 * 先月、今月、来月の3ヶ月カレンダーを表示する<br><br>
 *
 * <b> 処理詳細</b>
 * (1) 入力/システムから獲得した月によって以下の処理をする。<br>
 * (1.1) 月が1月なら、前年の12月からシステム上の年の2月までのカレンダーを作る。<br>
 * (1.2) 月が12月なら、11月から翌年の1月までのカレンダーを作る。<br>
 * (1.3) それ以外の月なら、前の月から翌月までのカレンダーを作る。<br>
 * (2) 3ヶ月分のカレンダーが作成したら、横にカレンダーを表示する<br>
 * <br>
 * 
 * @param year 年
 * @param month 月
 * @param today システム上の日付
 *
 * <b> 被参照自作関数</b><br>
 * ファイルghcal.c内  関数argcheck<br>
 * ファイルopreate_calendar内  関数this_three_month<br>
 * <br>
 *
 * <b> 参照自作関数</b><br>
 * ファイルopreate_calendar.c内　関数make_array<br>
 * ファイルcalendar_main.c内  関数make_three_calendar<br>
 * ファイルcalendar_main.c内  関数make_holiday<br>
 * ファイルoutput_calendar.c内  関数print_calendar_three<br>
 * ファイルoutput_calendar.c内  関数row_calendar_main<br>
 * ファイルoutput_calendar.c内  関数row_holiday<br>
 * <br>
 *
 */
void input_ym_three_today(int year, int month, DAY_DATE today) 
{
    /* ここから変数の宣言 */
	int cal_buf[COL_CAL][WEEK_ROW][WEEK_COL];			/* カレンダーのバッファ */
	char holiday_name[COL_CAL][HOLY_MAX][HOLY_NAME];	/* 月別の祝日 */ 
	int holiday_num[COL_CAL][HOLY_MAX];				/* 祝日用日付 */
	int hol_max = 0;										/* 3ヶ月間での最大祝日日数 */
	int week_max = 4;										/* 週数 */
	int i;
	int arr_year[COL_CAL];
	int arr_month[COL_CAL];
    
    /* ここから処理コードの記述 */
	make_array(arr_year, arr_month, year, month);  														/* ファイルopreate_calendar.c */
	make_three_calendar(arr_year, arr_month, cal_buf, &week_max);										/* ファイルcalendar_main.c */	
	for(i = 0; i < COL_CAL; i++){
		make_holiday(arr_year[i], arr_month[i], holiday_name[i], holiday_num[i], &hol_max);			/* ファイルcalendar_main.c */
	}
	print_calendar_three(arr_year, arr_month);															/* ファイルoutput_calendar.c */
	row_calendar_main_today(cal_buf, week_max, holiday_num, today.month, today.day, arr_month);	/* カレンダー表示(ファイルoutput_calendar.c) */
	row_holiday(holiday_name, hol_max);																	/* 祝日を表示(ファイルoutput_calendar.c) */
}


/*!
 * <b> 機能 </b><br>
 * システム上の年月日を取得する<br>
 * <br>
 *
 * @retval today 本日の日付が入った構造体型変数
 *
 * <b> 被参照自作関数</b><br>
 * ファイルopreate_calendar.c内 関数this_month_cal<br>
 * ファイルopreate_calendar.c内 関数next_month_cal<br>
 * ファイルopreate_calendar.c内 関数before_month_cal<br>
 * ファイルopreate_calendar.c内 関数this_year_cal<br>
 * ファイルopreate_calendar.c内 関数next_year_cal<br>
 * ファイルopreate_calendar.c内 関数before_year_cal<br>
 * ファイルopreate_calendar.c内 関数this_fiscal_cal<br>
 * ファイルopreate_calendar.c内 関数next_fiscal_cal<br>
 * ファイルopreate_calendar.c内 関数before_fiscal_cal<br>
 * ファイルopreate_calendar.c内 関数this_three_month<br>
 * ファイルopreate_calendar_gengoh.c内 関数this_month_cal_gengoh<br>
 * ファイルopreate_calendar_gengoh.c内 関数next_month_cal_gengoh<br>
 * ファイルopreate_calendar_gengoh.c内 関数before_month_cal_gengoh<br>
 * ファイルopreate_calendar_gengoh.c内 関数this_year_cal_gengoh<br>
 * ファイルopreate_calendar_gengoh.c内 関数next_year_cal_gengoh<br>
 * ファイルopreate_calendar_gengoh.c内 関数before_year_cal_gengoh<br>
 * ファイルopreate_calendar_gengoh.c内 関数this_fiscal_cal_gengoh<br>
 * ファイルopreate_calendar_gengoh.c内 関数next_fiscal_cal_gengoh<br>
 * ファイルopreate_calendar_gengoh.c内 関数before_year_cal_gengoh<br>
 * <br>
 * 
 * <b> 参照自作関数</b><br>
 * なし<br>
 *
 */
DAY_DATE get_today()
{
	/* 変数の宣言 */
	DAY_DATE today;		/* DAY_DATE型の変数を宣言 */
	time_t current;		/* 暦時刻型の変数宣言 */
	struct tm *timer;		/* 要素別の時刻 */

	/* ここから処理コードの記述 */
	time(&current);								/* 現在の時刻を取得する */
	timer = localtime(&current);				/* 要素別の時刻に変換する */
	today.year = (int) timer->tm_year + 1900;	/* 今年の年数を取得する */
	today.month = (int) timer->tm_mon + 1;		/* 今月の月数を取得する */
	today.day = (int) timer->tm_mday;			/* 今日の日付を取得する */
    
	return today;
}

/*!
 *<b> 機能 </b><br>
 * 配列にデータを入れる<br>
 *
 * @param arr_year[] 3ヶ月分の年
 * @param arr_month[] 3ヶ月分の月
 * @param year 基準年
 * @param month 基準月
 *
 * <b> 被参照自作関数</b><br>
 * ファイルopreate_calendar内 関数input_ym_three<br>
 * ファイルopreate_calendar内 関数input_ym_three_today<br>
 * <br>
 *
 * <b> 参照自作関数</b><br>
 * なし<br>
 * <br>
 *
 */
void make_array(int arr_year[COL_CAL], int arr_month[COL_CAL], int year, int month)
{
	/* ここから変数の宣言 */
	int i;
    
	/* ここから処理コードの記述 */
	for(i = 0; i < COL_CAL; i++){
		arr_year[i] = year;
		arr_month[i] = month + i - 1;
	}
	if(month == 1){
		arr_year[0] = year - 1;
		arr_month[0] = 12;
	}else if(month == 12){
		arr_year[2] = year + 1;
		arr_month[2]  = 1;
	}
}
