/*!
 * @file
 * @brief カレンダー操作プログラム(元号版)
 * @author y_okamon
 * @version 2.0(2011.07.01)<br>
 * 新ファイル作成の意図<br>
 * ファイルoutput_calendarの元号版を作成
 * <br>
 *
 */

/* ここからヘッダファイルの宣言 */
#include<stdio.h>
#include<string.h>
#include<stdlib.h>
#include<time.h>
#include"number_set.h"

/* ここから関数のプロトタイプの宣言 */
/* ファイルinsert_holiday.cから */
int judge_month(int year, int month, char holiday_name[HOLY_MAX][HOLY_NAME], int holiday_days[HOLY_MAX]);

/* ファイルopreate_calendar.c から */
DAY_DATE get_today();

/* ファイルoutput_calendar.cから */
void row_holiday(char holiday_name[COL_CAL][HOLY_MAX][HOLY_NAME], int hol_max);

/* ファイルopreate_calendar_gengoh.cから */
void year_month_gengoh_check(char chyear[10], char chmonth[10], int *year, int *month, int *gengoh);
void year_gengoh_check(char chmonth[10],  int *year, int *gengoh);
void opreate_year_gengoh(char chyear[10]);
void input_1989_cal_gengoh(int year);

/* ファイルoutput_calendar_gengoh.cから */
void month_print_calendar_h1_1(int year, int month);
void month_print_calendar_s64_1(int year, int month);
void print_calendar_three_1989(int year[COL_CAL], int month[COL_CAL]);

/* ファイルghcal.cから */
void month_check(int month);          /* 月の有効範囲を調べる */

/* ファイルerror_mes.cから */
void syowa_year_mes();
void heisei_year_mes();
void syowa_fiscal_mes();
void heisei_fiscal_mes();
void use_mes();

/* ファイルopreate_calendar_gengoh.cから */
void input_ym_three_gengoh(int year, int month);
void input_ym_three_today_gengoh(int year, int month, DAY_DATE today);
void input_year_cal_gengoh(int year);
void input_fiscal_cal_gengoh(int year);
void input_1989_three(int year, int month);
void input_1989_1to3(int year, int month);
void input_fiscal_1988(int year);
void input_fiscal_1989(int year);
void year_fiscal_check(char chyear[10], int *year, int *gengoh);
/* ファイルcalendar_main.cから */
void make_holiday_1989(int year, int month, char holiday_name[HOLY_MAX][HOLY_NAME], int holiday_num[HOLY_MAX], int *max_holidays);

/*!
 * <b> 機能 </b><br>
 * 今月のカレンダーを元号で表記する<br>
 * <br>
 *
 * <b> 処理詳細</b><br>
 * (1) システム上の年月を取得する。<br>
 * (2) 取得した年月を関数month_print_calendar_today_gengohで処理<br>
 * <br>
 *
 * <b> 被参照自作関数 </b><br>
 * ファイルghcal.c内  関数argcheck<br>
 * <br>
 *
 * <b> 参照自作関数 </b><br>
 * ファイルoutput_calendar.c内  関数month_print_calendar_today<br>
 * ファイルopreate_calendar.c内　関数get_today<br>
 * <br>
 */
void this_month_cal_gengoh() 
{
    /* ここから変数の宣言 */
	DAY_DATE today = get_today();	/* 現在の年月日を取得する(ファイルopreate_calendar.c) */
    
	/* ここから処理コードの記述 */
	printf("    平成%d年%2d月\n", today.year - 1988, today.month);		/* 年月を表示 */
	month_print_calendar_today(today.year, today.month, today.day);	/* 今月のカレンダーを表示する(ファイルoutput_calendar.c) */
}

/*!
 * <b> 機能</b><br>
 * 来月のカレンダーを元号で表示する<br>
 * <br>
 *
 * <b> 処理詳細</b><br>
 * (1) システム上の年月を取得する。<br>
 * (2) 取得した年月を関数month_print_calendarで処理<br>
 * <br>
 *
 * <b> 被参照自作関数 </b><br>
 * ファイルghcal.c内  関数argcheck<br>
 * <br>
 *
 * <b> 参照自作関数 </b><br>
 * ファイルoutput_calendar.c内  関数month_print_calendar<br>
 * ファイルopreate_calendar.c内　関数get_today<br>
 * <br>
 *
 */
void next_month_cal_gengoh() 
{
	/* ここから変数の宣言 */
	DAY_DATE today = get_today();	/* 日付けを取得する(ファイルopreate_calendar.c) */

	/* ここから処理コードの記述 */ 
	if(today.month == MONTH_MAX){													/* 今月が12月なら */
		printf("    平成%d年%2d月\n", today.year - 1987, MONTH_MIN);			/* 翌年の1月のカレンダーを表示 */
		month_print_calendar(today.year + 1, MONTH_MIN);						/* ファイルoutput_calendar.c */
	}else{																				/* それ以外 */
		printf("    平成%d年%2d月\n", today.year - 1988, today.month + 1);		/* 指定された年月のカレンダーを表示 */
		month_print_calendar(today.year, today.month + 1);						/* ファイルoutput_calendar.c */
    }
}

/*!
 * <b> 機能</b><br>
 * 先月のカレンダーを表示する<br>
 * <br>
 *
 * <b> 処理詳細</b><br>
 * (1) システム上の年月を取得する。<br>
 * (2) 取得した年月を関数month_print_calendarで処理<br>
 * <br>
 *
 * <b> 被参照自作関数 </b><br>
 * ファイルghcal.c内  関数argcheck<br>
 * <br>
 *
 * <b> 参照自作関数 </b><br>
 * ファイルoutput_calendar.c内  関数month_print_calendar<br>
 * ファイルopreate_calendar.c内 関数get_today<br>
 * <br>
 *
 */
void before_month_cal_gengoh() 
{
	/* ここから変数の宣言 */
	DAY_DATE today = get_today();	/* 日付を取得(ファイルopreate_calendar.c) */

	/* ここから処理コードの記述 */
	if(today.month == MONTH_MIN){													/* 今月が1月なら */
		printf("    平成%d年%2d月\n", today.year - 1989, MONTH_MAX);			/* 前年の12月のカレンダーを表示 */		
		month_print_calendar(today.year - 1, MONTH_MAX);						/* ファイルoutput_calendar.c */
	}else{																				/* それ以外 */
		printf("    平成%d年%2d月\n", today.year - 1988, today.month - 1);		/* 指定された年月のカレンダーを表示 */
		month_print_calendar(today.year, today.month - 1);						/* ファイルoutput_calendar.c */
	}
}


/*!
 * <b> 機能 </b><br>
 * year(元号形式)年month月のカレンダーを出すための操作を行う関数<br>
 * <br>
 *
 * @param chyear[] 年
 * @param chmonth[] 月
 *
 * <b> 被参照自作関数 </b><br>
 * ファイルghcal.c内 関数argcheck<br>
 * <br>
 * 
 * <b> 参照自作関数 </b><br>
 * ファイルopreate_calendar_gengoh.c内　関数year_month_gengoh_check<br>
 * ファイルoutput_calendar.c内　関数month_print_calendar<br>
 * ファイルoutput_calendar_gengoh.c内　関数month_print_calendar_s64_1<br>
 * ファイルoutput_calendar_gengoh.c内　関数month_print_calendar_h1_1<br>
 * <br>
 */
void opreate_year_month_gengoh(char chyear[10], char chmonth[10])
{
	/* ここから変数の宣言 */
	int year;		/* 西暦 */
	int month;		/* 月 */
	int gengoh;	/* 元号 */

	/* ここから処理コードの記述 */
	year_month_gengoh_check(chyear, chmonth, &year, &month, &gengoh);		/* ファイルopreate_calendar_gengoh.c */
	if(year == 1989  && gengoh == 1 && month >= 2){
		printf("    平成元年%2d月\n", month);									/* 指定された年月のカレンダーを表示 */
		month_print_calendar(year, month);										/* ファイルoutput_calendar.c */
	}else if(year == 1989 && gengoh == 1 && month == 1){
		printf("    平成元年 1月\n");
		month_print_calendar_h1_1(year, month);								/* ファイルoutput_calendar_gengoh.c */ 
	}else if(year == 1989 && gengoh == 64 && month == 1){
		printf("    昭和64年 1月\n");
		month_print_calendar_s64_1(year, month);								/* ファイルoutput_calendar_gengoh.c */ 
	}else if(year > 1989){
		printf("    平成%d年%2d月\n", gengoh, month);							/* 指定された年月のカレンダーを表示 */
		month_print_calendar(year, month);										/* ファイルoutput_calendar.c */
	}else if(year < 1989){
		printf("    昭和%d年%2d月\n", gengoh, month);							/* 指定された年月のカレンダーを表示 */
		month_print_calendar(year, month);										/* ファイルoutput_calendar.c */
	}	
}

/*!
 * <b> 機能 </b><br>
 * year(元号形式)年のカレンダーを出すための操作を行う関数<br>
 * <br>
 *
 * @param chyear[] 年
 *
 * <b> 被参照自作関数 </b><br>
 * ファイルghcal.c内 関数argcheck<br>
 * <br>
 * 
 * <b> 参照自作関数 </b><br>
 * ファイルopreate_calendar_gengoh.c内　関数year_gengoh_check<br>
 * ファイルoutput_calendar_gengoh.c内　関数month_print_calendar_s64_1<br>
 * ファイルopreate_calendar_gengoh.c内　関数input_year_cal_gengoh<br>
 * ファイルopreate_calendar_gengoh.c内 関数input_1989_cal_gengoh<br>
 * <br>
 */
void opreate_year_gengoh(char chyear[10])
{
	/* ここから変数の宣言 */
	int year;		/* 西暦 */
	int gengoh;	/* 元号 */

	/* ここから処理コードの記述 */
	year_gengoh_check(chyear, &year, &gengoh);									/* ファイルopreate_calendar_gengoh.c */
	if(year == 1989  && gengoh == 1 ){
		printf("                              平成元年\n\n");
		input_1989_cal_gengoh(year);												/* ファイルopreate_calendar_gengoh.c */
	}else if(year == 1989 && gengoh == 64){
		printf("                              昭和%2d年\n\n", year - 1925);	
		printf("    昭和64年 1月\n");
		month_print_calendar_s64_1(year, 1);  									/* ファイルoutput_calendar_gengoh.c */
	}else if(year != 1989){
		input_year_cal_gengoh(year);												/* ファイルopreate_calendar_gengoh.c */
	}
}

/*!
 * <b> 機能 </b><br>
 * 今年の年間カレンダーを表示する<br>
 * <br>
 *
 * <b> 処理詳細 </b><br>
 * (1) システム上の年を取得する<br>
 * (2) 取得した年をinput_year_cal関数で処理<br>
 * <br>
 * 
 * <b> 被参照自作関数</b><br>
 * ファイルghcal.c内 関数argcheck<br>
 * <br>
 *
 * <b> 参照自作関数</b><br>
 * ファイルoutput_calendar_gengoh.c内  関数input_ym_three_today_gengoh<br>
 * ファイルoutput_calendar_gengoh.c内　関数input_ym_three_gengoh<br>
 * ファイルoutput_calendar.c内　関数get_today<br>
 * <br>
 *
 */
void this_year_cal_gengoh()
{
    /* ここから変数の宣言 */
	DAY_DATE today = get_today();   /* 日付データ構造体変数(ファイルopreate_calendar.c) */
	int i;
	int month = 0;
    
	/* ここから処理コードの記述 */
	printf("                              平成%2d年\n\n", today.year - 1988);						/* 先頭にxxxx年のカレンダーを表示させる(Ver2.0) */
	for(i = 0; i < 4; i++){
		month = month + 3;
		if(today.month == month - 2 || today.month == month - 1 || today.month == month){		
			input_ym_three_today_gengoh(today.year, month - 1, today);								/* ファイルopreate_calendar_gengoh.c */
		}else{
			input_ym_three_gengoh(today.year, month -1);												/* ファイルopreate_calendar_gengoh.c */
		}
	}
}

/*!
 * <b> 機能 </b><br>
 * 翌年の年間カレンダーを表示する<br>
 * <br>
 *
 * <b> 処理詳細 </b><br>
 * (1) システム上の年を取得する<br>
 * (2) 取得した年+1をinput_year_cal_gengoh関数で処理<br>
 * <br>
 * 
 * <b> 被参照自作関数</b><br>
 * ファイルghcal.c内 関数argcheck<br>
 * <br>
 *
 * <b> 参照自作関数</b><br>
 * ファイルopreate_calendar_gengoh.c内  関数input_year_cal_gengoh<br>
 * ファイルopreate_calendar.c内　関数get_today<br>
 * <br>
 * 
 */
void next_year_cal_gengoh() 
{
	/* ここから変数の宣言 */
	DAY_DATE today = get_today();	/* 本日の日付を取得(ファイルopreate_calendar.c) */
    
	/* ここから処理コードの記述 */
	input_year_cal_gengoh(today.year + 1);	/* カレンダーを表示する(ファイルopreate_calendar_gengoh.c) */
}

/*!
 * <b> 機能 </b><br>
 * 昨年の年間カレンダーを表示する<br>
 * <br>
 *
 * <b> 処理詳細 </b><br>
 * (1) システム上の年を取得する<br>
 * (2) 取得した年から１を引いた年をinput_year_cal_gengoh関数で処理<br>
 * <br>
 * 
 * <b> 被参照自作関数</b><br>
 * ファイルghcal.c内 関数argcheck<br>
 * <br>
 *
 * <b> 参照自作関数</b><br>
 * ファイルopreate_calendar_gengoh.c内  関数input_year_cal_gengoh<br>
 * ファイルopreate_calendar.c内　関数get_today<br>
 * <br>
 * 
 */
void before_year_cal_gengoh() 
{
	/* ここから変数の宣言 */
	DAY_DATE today = get_today();	/* 本日の日付を取得する(ファイルopreate_calendar.c) */

	/* ここから処理コードの記述 */
	input_year_cal_gengoh(today.year - 1);	/* カレンダーを表示する(ファイルopreate_calendar_gengoh.c) */
}

/*!
 * <b> 機能</b><br>
 * 入力された年の年間カレンダーを表示させる<br>
 * <br>
 *
 * <b> 処理詳細</b><br>
 * 横に3ヶ月分のカレンダーを表示する。<br>
 * <br>
 *
 * @param year 年
 *  
 * <b> 被参照自作関数</b><br>
 * ファイルopreate_calendar_gengoh.c内  関数next_year_cal_gengoh<br>
 * ファイルopreate_calendar_gengoh.c内　関数before_year_cal_gengoh<br>
 * ファイルopreate_calendar_gengoh内  関数opreate_year_gengoh<br>
 * <br>
 *
 * <b> 参照自作関数</b><br>
 * ファイルoutput_calendar_gengoh.c内  関数input_ym_three_gengoh<br>
 * <br>
 *
 */
void input_year_cal_gengoh(int year) 
{
	/* ここから変数の宣言 */
	int month = 1;	/* 月 */
	int i;				/* ループ変数 */

	/* ここから処理コードの記述 */
	if(year > 1989){
		printf("                              平成%2d年\n\n", year - 1988);	/* 先頭にxxxx年のカレンダーを表示させる(Ver2.0) */
	}else if(year < 1989){
		printf("                              昭和%2d年\n\n", year - 1925);	/* 先頭にxxxx年のカレンダーを表示させる(Ver2.0) */	
	}	
	for(i = 0; i < 4; i++){															/* 3ヶ月ごと4回 */
		month = 3 * i + 2;															/* 2,5,8,11月を基準とする */
		input_ym_three_gengoh(year, month);										/* 3ヶ月カレンダーを表示(ファイルopreate_calendar_gengoh.c) */
	}
}


/*!
 * <b> 機能</b><br>
 * 平成元年(1989年)年間カレンダーを表示させる<br>
 * <br>
 *
 * @param year 年
 *  
 * <b> 被参照自作関数</b><br>
 * ファイルopreate_calendar_gengoh内  関数opreate_year_gengoh<br>
 * <br>
 *
 * <b> 参照自作関数</b><br>
 * ファイルopreate_calendar_gengoh.c内  関数input_1989_1to3<br>
 * ファイルopreate_calendar_gengoh.c内  関数input_1989_three<br>
 * <br>
 *
 */
void input_1989_cal_gengoh(int year) 
{
	/* ここから変数の宣言 */
	int month = 2;	/* 月 */
	int i;				/* ループ変数 */

	/* ここから処理コードの記述 */
	input_1989_1to3(year, month);			/* 1月から3月まで(ファイルopreate_calendar_gengoh.c) */ 		
	for(i = 1; i < 4; i++){					/* 3ヶ月ごと4回 */
		month = 3 * i + 2;					/* 5,8,11月を基準とする */
		input_1989_three(year, month);		/* 3ヶ月カレンダーを表示(ファイルorpeate_calendar_gengoh.c) */
	}
}

/*!
 * <b> 機能</b>
 * 平成元年の表示する<br><br>
 * 
 * @param year 年
 * @param month 年
 *
 * <b> 被参照自作関数</b><br>
 * ファイルopreate_calendar_gengoh内  関数input_1989_cal_gengoh<br>
 * <br>
 *
 * <b> 参照自作関数</b><br>
 * ファイルcalendar_main.c内 関数make_array<br>
 * ファイルcalendar_main.c内  関数make_three_calendar_1989<br>
 * ファイルcalendar_main.c内　関数make_holiday_1989<br>
 * ファイルoutput_calendar_gengoh.c内  関数print_calendar_three_1989<br>
 * ファイルoutput_calendar.c内  関数row_calendar_main<br>
 * ファイルoutput_calendar.c内  関数row_holiday<br>
 * <br>
 *
 */
void input_1989_1to3(int year, int month) 
{
	/* ここから変数の宣言 */
	int cal_buf[COL_CAL][WEEK_ROW][WEEK_COL];				/* カレンダーのバッファ */
	char holiday_name[COL_CAL][HOLY_MAX][HOLY_NAME];	/* 月別の祝日 */ 
	int holiday_num[COL_CAL][HOLY_MAX];					/* 祝日用日付 */
	int hol_max = 0;											/* 3ヶ月間での最大祝日日数 */
	int week_max = 4;											/* 週数 */
	int i;
	int arr_year[COL_CAL];
	int arr_month[COL_CAL];
    
	/* ここから処理コードの記述 */
	make_array(arr_year, arr_month, year, month);															/* ファイルopreate_calendar.c */
	make_three_calendar_1989(arr_year, arr_month, cal_buf, &week_max);									/* ファイルcalendar_main.c */
	for(i = 0; i < COL_CAL; i++){
		make_holiday_1989(arr_year[i], arr_month[i], holiday_name[i], holiday_num[i], &hol_max);		/* ファイルcalendar_main.c */
	}
	print_calendar_three_1989(arr_year, arr_month);															/* ファイルoutput_calendar_gengoh.c */
	row_calendar_main(cal_buf, week_max, holiday_num);														/* カレンダーを表示する(ファイルoutput_calendar.c) */
	row_holiday(holiday_name, hol_max);																		/* 祝日を表示(ファイルoutput_calendar.c) */
}

/*!
 * <b> 機能</b>
 * 1989年の4月～12月までのの3ヶ月カレンダーを表示する<br><br>
 * 
 * @param year 年
 * @param month 年
 *
 * <b> 被参照自作関数</b><br>
 * ファイルoutput_calendar_gengoh.c内　関数input_cal_1989_gengoh<br>
 * ファイルoutput_calendar_gengoh.c内　関数input_fiscal_1988<br>
 * ファイルoutput_calendar_gengoh.c内　関数input_fiscal_1989<br>
 * <br>
 *
 * <b> 参照自作関数</b><br>
 * ファイルcalendar_main.c内 関数make_array<br>
 * ファイルcalendar_main.c内  関数make_three_calendar<br>
 * ファイルcalendar_main.c内　関数make_holiday<br>
 * ファイルoutput_calendar_gengoh.c内  関数print_calendar_three_1989<br>
 * ファイルoutput_calendar.c内  関数row_calendar_main<br>
 * ファイルoutput_calendar.c内  関数row_holiday<br>
 * <br>
 *
 */
void input_1989_three(int year, int month) 
{
	/* ここから変数の宣言 */
	int cal_buf[COL_CAL][WEEK_ROW][WEEK_COL];				/* カレンダーのバッファ */
	char holiday_name[COL_CAL][HOLY_MAX][HOLY_NAME];	/* 月別の祝日 */ 
	int holiday_num[COL_CAL][HOLY_MAX];             	/* 祝日用日付 */
	int hol_max = 0;                                	/* 3ヶ月間での最大祝日日数 */
	int week_max = 4;                               	/* 週数 */
	int i;
	int arr_year[COL_CAL];
	int arr_month[COL_CAL];

	/* ここから処理コードの記述 */
	make_array(arr_year, arr_month, year, month);													/* ファイルopreate_calendar.c */
	make_three_calendar(arr_year, arr_month, cal_buf, &week_max);									/* ファイルcalendar_main.c */
	for(i = 0; i < COL_CAL; i++){
		make_holiday(arr_year[i], arr_month[i], holiday_name[i], holiday_num[i], &hol_max);	/* ファイルcalendar_main.c */
	}
	print_calendar_three_1989(arr_year, arr_month);													/* ファイルoutput_calendar_gengoh.c */
	row_calendar_main(cal_buf, week_max, holiday_num);                                       /* カレンダーを表示する(ファイルoutput_calendar.c) */
	row_holiday(holiday_name, hol_max);                                                      /* 祝日を表示(ファイルoutput_calendar.c) */
}

/*!
 * <b> 機能 </b><br>
 * year(元号形式)年のカレンダーを出すための操作を行う関数<br>
 * <br>
 *
 * @param chyear[] 年
 *
 * <b> 被参照自作関数 </b><br>
 * ファイルghcal.c内 関数argcheck<br>
 * <br>
 * 
 * <b> 参照自作関数 </b><br>
 * ファイルopreate_calendar_gengoh.c内　関数year_fiscal_check<br>
 * ファイルopreate_calendar_gengoh.c内　関数input_fiscal_1989<br>
 * ファイルopreate_calendar_gengoh.c内  関数input_fiscal_1988<br>
 * ファイルopreate_calendar_gengoh.c内  関数input_fiscal_gengoh<br>
 * <br>
 */
void opreate_fiscal_gengoh(char chyear[10])
{
	/* ここから変数の宣言 */
	int year;		/* 西暦 */
	int gengoh;	/* 元号 */

	/* ここから処理コードの記述 */
	year_fiscal_check(chyear, &year, &gengoh);						/* ファイルopreate_calendar_gengoh.c */
	if(year == 1989){
		printf("                              平成元年度\n\n");
		input_fiscal_1989(year);										/* ファイルopreate_calendar_gengoh.c */
	}else if(year == 1988){
		printf("                              昭和63年度\n\n");
		input_fiscal_1988(year);										/* ファイルopreate_calendar_gengoh.c */
	}else{
		input_fiscal_cal_gengoh(year);									/* ファイルopreate_calendar_gengoh.c */
	}
}


/*!
 * <b>機能</b><br>
 * 今年度のカレンダーを表示する<br><br>
 *
 * <b> 処理詳細 </b><br>
 * (1) システム上の年と月を取得する<br>
 * (2) 取得した月が1月〜3月の間であったら、年を1年マイナス<br>
 * (3) 取得した年をinput_year_cal関数で処理<br>
 * <br>
 * 
 * <b> 被参照自作関数</b><br>
 * ファイルghcal.c内 関数argcheck<br>
 * <br>
 *
 * <b> 参照自作関数</b><br>
 * ファイルopreate_calendar_gengoh.c内  関数input_ym_three_today_gengoh<br>
 * ファイルopreate_calendar_gengoh.c内  関数input_ym_three_gengoh<br>
 * ファイルopreate_calendar.c内　関数get_today<br>
 * <br>
 * 
 */
void this_fiscal_cal_gengoh() 
{
    /* ここから変数の宣言 */
	DAY_DATE today = get_today();	/* 日付の構造体(ファイルopreate_calendar.c) */
	int month = 3;
	int i;
    
	/* ここから処理コードの記述 */
	if(today.month < 4){																					/* 現在の月が1月〜3月まで(Ver1.01) */
		today.year--;																						/* 年度はyear -1 年度(Ver1.01) */
	}
	printf("                              平成%2d年\n\n", today.year - 1988);						/* 先頭にxxxx年のカレンダーを表示させる(Ver2.0) */	
	for(i = 0; i < 3; i++){																				/* 4月〜12月まで */
		month = month + 3;																				/* 3ヶ月更新 */
		if(today.month == month - 2 || today.month == month - 1 || today.month == month){		
			input_ym_three_today_gengoh(today.year, month - 1, today);								/* ファイルopreate_calendar_gengoh.c */
		}else{
			input_ym_three_gengoh(today.year, month -1);												/* ファイルopreate_calendar_gengoh.c */
		}
	}
	month = 3;
	if(today.month == month - 2 || today.month == month - 1 || today.month == month){
		input_ym_three_today_gengoh(today.year + 1, month - 1, today);							/* ファイルopreate_calendar_gengoh.c */
	}else{
		input_ym_three_gengoh(today.year + 1, month -1);											/* ファイルopreate_calendar_gengoh.c */
	} 
}

/*!
 * <b>機能</b><br>
 * 翌年度のカレンダーを表示する<br><br>
 *
 * <b> 処理詳細 </b><br>
 * (1) システム上の年と月を取得する<br>
 * (2) 年をプラス1する。<br>
 * (3) 取得した月が1月〜3月の間であったら、年を1年マイナス<br>
 * (4) 取得した年をinput_year_cal関数で処理<br>
 * <br>
 * 
 * <b> 被参照自作関数</b><br>
 * ファイルghcal.c内 関数argcheck<br>
 * <br>
 *
 * <b> 参照自作関数</b><br>
 * ファイルopreate_calendar_gengoh.c内  関数input_fiscal_cal_gengoh<br>
 * ファイルopreate_calendar内　関数get_today<br>
 * <br>
 */
void next_fiscal_cal_gengoh() 
{
	/* ここから変数の宣言 */
	DAY_DATE today = get_today();	/* 日付を取得する(ファイルopreate_calendar.c) */

	/* ここから処理コードの記述 */
	if (today.month < 4) {							/* 現在の月が1月〜3月まで(Ver1.01) */
		today.year--;									/* 年度はyear -1 年度(Ver1.01) */
	}
	input_fiscal_cal_gengoh(today.year + 1);		/* カレンダーを表示する(ファイルopreate_calendar_gengoh.c) */
}


/*!
 * <b>機能</b><br>
 * 前年度のカレンダーを表示する<br><br>
 *
 * <b> 処理詳細 </b><br>
 * (1) システム上の年と月を取得する<br>
 * (2) 年をマイナス1する。
 * (3) 取得した月が1月〜3月の間であったら、年を1年マイナス<br>
 * (4) 取得した年をinput_year_cal関数で処理<br>
 * <br>
 * 
 * <b> 被参照自作関数</b><br>
 * ファイルghcal.c内 関数argcheck<br>
 * <br>
 *
 * <b> 参照自作関数</b><br>
 * ファイルopreate_calendar_gengoh内  関数input_fiscal_cal_gengoh<br>
 * ファイルopreate_calendar.c内  関数get_today<br>
 * <br>
 * 
 */
void before_fiscal_cal_gengoh() 
{
	/* ここから変数の宣言 */
	DAY_DATE today = get_today();	/* 今日の日付を取得する(ファイルopreate_calendar.c) */

	/* ここから処理コードの記述 */
	if (today.month < 4) {							/* 現在の月が1月〜3月まで */
		today.year --;								/* 年度はyear -1 年度(Ver1.01) */
	}
	input_fiscal_cal_gengoh(today.year - 1);		/* カレンダーを表示する(ファイルopreate_calendar_gengoh.c) */
}
 

/*!
 * <b> 機能</b><br>
 * 入力された年の年度カレンダーを表示させる<br>
 * <br>
 *
 * @param year 年
 *
 * <b> 被参照自作関数</b><br>
 * ファイルopreate_calendar_gengoh.c内  関数next_fiscal_cal_gengoh<br>
 * ファイルopreate_calendar_gengoh.c内　関数before_fiscal_cal_gengoh<br>
 * ファイルopreate_calendar_gengoh.c内  関数opreate_fiscal_gengoh<br>
 * <br>
 *
 * <b> 参照自作関数</b><br>
 * ファイルopeate_calendar_gengoh.c内  関数input_ym_three_gengoh<br>
 * <br>
 * 
 */
void input_fiscal_cal_gengoh(int year) 
{
	/* ここから変数の宣言 */
	int month = 4;	/* 月 */
	int i;				/* ループ変数 */    

	/* ここから処理コードの記述 */
	if(year > 1989){
		printf("                            平成%2d年度\n\n", year - 1988);	/* 先頭にxxxx年のカレンダーを表示させる(Ver2.0) */
	}else if(year < 1989){
		printf("                            昭和%2d年度\n\n", year - 1925);	/* 先頭にxxxx年のカレンダーを表示させる(Ver2.0) */	
	}	
	for(i = 1; i < 4; i++){ 														/* 4月〜12月まで */
		month = 3 * i + 2;															/* 中央にくる月を算出 */
		input_ym_three_gengoh(year, month);										/* カレンダーを表示(ファイルopreate_calendar_gengoh.c) */
	}
	input_ym_three_gengoh(year + 1, 2);											/* ファイルopreate_calendar_gengoh.c */
}

/*!
 * <b> 機能 </b><br>
 * 昭和63年度のカレンダーを表示する<br>
 * <br>
 *
 * <b> 被参照自作関数 </b><br>
 * ファイルopreate_calendar_gengoh.c内  関数opreate_fiscal_gengoh<br>
 * <br>
 *
 * <b> 参照自作関数 </b><br>
 * ファイルopreate_calendar_gengoh.c内　関数input_ym_three_gengoh<br>
 * ファイルopreate_calendar_gengoh.c内  関数input_1989_three<br>
 * <br>
 *
 */
void input_fiscal_1988(int year)
{
	/* ここから変数の宣言 */
	int month = 4;
	int i;

	/* ここから処理コードの記述 */
	for(i = 1; i < 4; i++){						/* 4月〜12月まで */
		month = 3 * i + 2;						/* 中央にくる月を算出 */
		input_ym_three_gengoh(year, month);	/* カレンダーを表示(ファイルopreate_calendar_gengoh.c) */
	}
	input_1989_three(year + 1, 2); 			/* ファイルopreate_calendar_main.c */
}

/*!
 * <b> 機能 </b><br>
 * 平成元年度のカレンダーを表示する<br>
 * <br>
 *
 * <b> 被参照自作関数 </b><br>
 * ファイルopreate_calendar_gengoh.c内  関数opreate_fiscal_gengoh<br>
 * <br>
 *
 * <b> 参照自作関数 </b><br>
 * ファイルopreate_calendar_gengoh.c内  関数input_1989_three<br>
 * ファイルopreate_calendar_gengoh.c内  関数input_ym_three_gengoh<br>
 * <br>
 *
 */
void input_fiscal_1989(int year)
{
	/* ここから変数の宣言 */
	int month = 4;
	int i;

	/* ここから処理コードの記述 */
	for(i = 1; i < 4; i++){					/* 4月〜12月まで */
		month = 3 * i + 2;					/* 中央にくる月を算出 */
		input_1989_three(year, month);		/* カレンダーを表示(ファイルopreate_calendar_gengoh.c) */
	}
	input_ym_three_gengoh(year + 1, 2);	/* ファイルopreate_calendar_gengoh.c */
}

/*!
 * <b> 機能 </b><br>
 * 入力されたコマンドの利用範囲が正しいかどうか調べる<br>
 * <br>
 *
 * @param chyear[] 年
 * @param chmonth[] 月
 * @param *year 数値化された西暦
 * @param *month 数値化された月
 * @param *gengoh 数値化された元号
 *
 * <b> 被参照自作関数</b><br>
 * ファイルopreate_calendar_gengoh.c内  関数opreate_year_month_gengoh<br>
 * <br>
 *
 * <b> 参照自作関数</b><br>
 * ファイルerror_mes.c内　関数syowa_year_mes<br>
 * ファイルerror_mes.c内　関数heisei_year_mes<br>
 * ファイルghcal.c内　関数month_check<br>
 * <br>
 */
void year_month_gengoh_check(char chmonth[10], char chyear[10], int *year, int *month, int *gengoh)
{
	/* ここから変数の宣言 */
	char chgengoh[10];	/* 元号 */
	int i = 1;
	int j = 0;
	
	/* ここから処理コードの記述 */
	while((chgengoh[j] = chyear[i]) != '\0'){					/* s45やh23と入力された場合 */
		i++;
		j++;
	}
	*gengoh = (int)atoi(chgengoh);								/* 元号を数値化する */
	*month = (int)atoi(chmonth);  								/* 入力された引数を数値に変換する */
	month_check(*month);	          								/* 月の有効範囲を調べる(ファイルghcal.c) */
	if(chyear[0] == 's'){										/* 先頭文字がs */
		if(*gengoh >= SYOWA_MIN && *gengoh <= SYOWA_MAX){	/* 昭和24年〜昭和64年まで */
			*year = 1925 + (*gengoh);							/* 西暦変換する */
		}else{														/* それ以外の昭和元号 */
			syowa_year_mes();										/* 昭和の利用範囲を表示(ファイルerror_mes.c) */
		}
		if(*gengoh == 64 && *month >= 2){						/* 昭和64年は1月7日まで */
			printf("昭和64年%2d月は存在しません\n", *month);
		}
	}else if(chyear[0] == 'h'){									/* 先頭文字がh */
		if(*gengoh >= HEISEI_MIN && *gengoh <= HEISEI_MAX){	/* 平成元年〜平成99年まで */
			*year = 1988 + (*gengoh);							/* 西暦変換 */
		}else{														/* 平成100年以降 */	
			heisei_year_mes();									/* 平成の利用範囲を表示(ファイルerror_mes.c) */
		}
	}
}

/*!
 * <b> 機能 </b><br>
 * 入力された元号年度の使用範囲をチェックする<br>
 * <br>
 *
 * @param chyear 年
 * @param *year 数値化された西暦
 * @param *gengoh 数値化された元号
 *
 * <b> 被参照自作関数 </b><br>
 * ファイルopreate_calendar_gengoh.c内　関数opreate_fiscal_gengoh<br>
 * <br>
 *
 * <b> 参照自作関数 </b><br>
 * ファイルerror_mes.c内  関数showa_fiscal_mes<br>
 * ファイルerror_mes.c内  関数heisei_fiscal_mes<br>
 * <br>
 *
 */
void year_fiscal_check(char chyear[10], int *year, int *gengoh)
{
	/* ここから変数の宣言 */
	char chgengoh[10];	/* 元号 */
	int i = 1;
	int j = 0;
	
	/* ここから処理コードの記述 */
	while((chgengoh[j] = chyear[i]) != '\0'){					/* s45やh23と入力された場合 */
		i++;
		j++;
	}
	*gengoh = (int)atoi(chgengoh);								/* 元号を数値化する */
	if(chyear[0] == 's'){										/* 先頭文字がs */
		if(*gengoh >= SYOWA_MIN && *gengoh < SYOWA_MAX){	/* 昭和24年〜昭和63年まで */
			*year = 1925 + (*gengoh);							/* 西暦変換する */
		}else{														/* それ以外の昭和元号 */
			syowa_fiscal_mes();									/* 昭和の利用範囲を表示(後で元号版を作成 ファイルerror_mes.c) */
			exit(1);												/* ヘッダファイルstdlib.h */
		}
	}else if(chyear[0] == 'h'){									/* 先頭文字がh */
		if(*gengoh >= HEISEI_MIN && *gengoh < HEISEI_MAX){	/* 平成元年〜平成98年まで */
			*year = 1988 + (*gengoh);							/* 西暦変換 */
		}else{														/* 平成99年以降 */	
			heisei_fiscal_mes();									/* 平成の利用範囲を表示(ファイルerror_mes.c) */
			exit(1);												/* ヘッダファイルstdlib.h */
		}
	}else{
		use_mes();													/* 使い方を説明(ファイルerror_mes.c) */
		exit(1);												
	}
}

/*!
 * <b> 機能 </b><br>
 * 入力されたコマンドの利用範囲が正しいかどうか調べる<br>
 * <br>
 *
 * @param chyear[] 年
 * @param *year 数値化された西暦
 * @param *gengoh 数値化された元号
 *
 * <b> 被参照自作関数</b><br>
 * ファイルopreate_calendar_gengoh.c内  関数opreate_year_gengoh<br>
 * <br>
 *
 * <b> 参照自作関数</b><br>
 * ファイルerror_mes.c内　関数syowa_year_mes();
 * ファイルerror_mes.c内　関数heisei_year_mes();
 * <br>
 */
void year_gengoh_check(char chyear[10],  int *year, int *gengoh)
{
	/* ここから変数の宣言 */
	char chgengoh[10];	/* 元号 */
	int i = 1;
	int j = 0;
	
	/* ここから処理コードの記述 */
	while((chgengoh[j] = chyear[i]) != '\0'){					/* s45やh23と入力された場合 */
		i++;
		j++;
	}
	*gengoh = (int)atoi(chgengoh);								/* 元号を数値化する */
	if(chyear[0] == 's'){										/* 先頭文字がs */
		if(*gengoh >= SYOWA_MIN && *gengoh <= SYOWA_MAX){	/* 昭和24年〜昭和64年まで */
			*year = 1925 + (*gengoh);							/* 西暦変換する */
		}else{														/* それ以外の昭和元号 */
			syowa_year_mes();										/* 昭和の利用範囲を表示 */
			exit(1);
		}
	}else if(chyear[0] == 'h'){									/* 先頭文字がh */
		if(*gengoh >= HEISEI_MIN && *gengoh <= HEISEI_MAX){	/* 平成元年〜平成99年まで */
			*year = 1988 + (*gengoh);							/* 西暦変換 */
		}else{														/* 平成100年以降 */	
			heisei_year_mes();									/* 平成の利用範囲を表示 */
			exit(1);
		}
	}else{
		use_mes();													/* 使い方を表示(ファイルerror_mes.c) */									
		exit(1);	
	}
}

/*!
 * <b> 機能</b>
 * 先月、今月、来月の3ヶ月カレンダーを表示する(元号版)<br><br>
 *
 * <b> 処理詳細</b>
 * (1) 入力/システムから獲得した月によって以下の処理をする。<br>
 * (1.1) 月が1月なら、前年の12月からシステム上の年の2月までのカレンダーを作る。<br>
 * (1.2) 月が12月なら、11月から翌年の1月までのカレンダーを作る。<br>
 * (1.3) それ以外の月なら、前の月から翌月までのカレンダーを作る。<br>
 * (2) 3ヶ月分のカレンダーが作成したら、横にカレンダーを表示する<br>
 * <br>
 * 
 * @param year 年
 * @param month 年
 *
 * <b> 被参照自作関数</b><br>
 * ファイルopreate_calendar_gengoh.c内  関数input_year_cal_gengoh<br>
 * ファイルopreate_calendar_gengoh.c内  関数input_fiscal_cal_gengoh<br>
 * ファイルopreate_calendar_gengoh.c内  関数input_fiscal_1988<br>
 * ファイルopreate_calendar_gengoh.c内  関数input_fiscal_1989<br>
 * <br>
 *
 * <b> 参照自作関数</b><br>
 * ファイルopreate_calendar.c内 関数make_array<br>
 * ファイルcalendar_main.c内  関数make_three_calendar<br>
 * ファイルcalendar_main.c内　関数make_holiday<br>
 * ファイルoutput_calendar_gengoh.c内  関数print_calendar_three_gengoh<br>
 * ファイルoutput_calendar.c内  関数row_calendar_main<br>
 * ファイルoutput_calendar.c内  関数row_holiday<br>
 * <br>
 *
 */
void input_ym_three_gengoh(int year, int month) 
{
	/* ここから変数の宣言 */
	int cal_buf[COL_CAL][WEEK_ROW][WEEK_COL];				/* カレンダーのバッファ */
	char holiday_name[COL_CAL][HOLY_MAX][HOLY_NAME];	/* 月別の祝日 */ 
	int holiday_num[COL_CAL][HOLY_MAX];					/* 祝日用日付 */
	int hol_max = 0;											/* 3ヶ月間での最大祝日日数 */
	int week_max = 4;											/* 週数 */
	int i;
	int arr_year[COL_CAL];
	int arr_month[COL_CAL];
    
    /* ここから処理コードの記述 */
	make_array(arr_year, arr_month, year, month);													/* ファイルopeate_calendar.c */
	make_three_calendar(arr_year, arr_month, cal_buf, &week_max);									/* ファイルcalendar_main.c */
	for(i = 0; i < COL_CAL; i++){
		make_holiday(arr_year[i], arr_month[i], holiday_name[i], holiday_num[i], &hol_max);	/* ファイルcalendar_main.c */
	}
	print_calendar_three_gengoh(arr_year, arr_month);												/* ファイルoutput_calendar_gengoh.c */
	row_calendar_main(cal_buf, week_max, holiday_num);												/* カレンダーを表示する(ファイルoutput_calendar_gengoh.c) */
	row_holiday(holiday_name, hol_max);																/* 祝日を表示(ファイルoutput_calendar.c) */
}

/*!
 * <b> 機能</b>
 * 先月、今月、来月の3ヶ月カレンダーを表示する(元号版)<br><br>
 *
 * <b> 処理詳細</b>
 * (1) 入力/システムから獲得した月によって以下の処理をする。<br>
 * (1.1) 月が1月なら、前年の12月からシステム上の年の2月までのカレンダーを作る。<br>
 * (1.2) 月が12月なら、11月から翌年の1月までのカレンダーを作る。<br>
 * (1.3) それ以外の月なら、前の月から翌月までのカレンダーを作る。<br>
 * (2) 3ヶ月分のカレンダーが作成したら、横にカレンダーを表示する<br>
 * <br>
 * 
 * @param year 年
 * @param month 月
 * @param today システム上の日付
 *
 * <b> 被参照自作関数</b><br>
 * ファイルopreate_calendar_gengoh.c内  関数this_year_gengoh<br>
 * ファイルopreate_calendar_gengoh.c内  関数this_fiscal_gengoh<br>
 * <br>
 *
 * <b> 参照自作関数</b><br>
 * ファイルopreate_calendar.c内　関数make_array<br>
 * ファイルcalendar_main.c内  関数make_three_calendar<br>
 * ファイルcalendar_main.c内  関数make_holiday<br>
 * ファイルoutput_calendar_gengoh.c内  関数print_calendar_three_gengoh<br>
 * ファイルoutput_calendar.c内  関数row_calendar_main<br>
 * ファイルoutput_calendar.c内  関数row_holiday<br>
 * <br>
 *
 */
void input_ym_three_today_gengoh(int year, int month, DAY_DATE today) 
{
    /* ここから変数の宣言 */
	int cal_buf[COL_CAL][WEEK_ROW][WEEK_COL];				/* カレンダーのバッファ */
	char holiday_name[COL_CAL][HOLY_MAX][HOLY_NAME];	/* 月別の祝日 */ 
	int holiday_num[COL_CAL][HOLY_MAX];					/* 祝日用日付 */
	int hol_max = 0;											/* 3ヶ月間での最大祝日日数 */
	int week_max = 4;											/* 週数 */
	int i;
	int arr_year[COL_CAL];
	int arr_month[COL_CAL];
    
	/* ここから処理コードの記述 */
	make_array(arr_year, arr_month, year, month);														/* ファイルopreate_calendar.c */
	make_three_calendar(arr_year, arr_month, cal_buf, &week_max);										/* ファイルcalendar_main.c */
	for(i = 0; i < COL_CAL; i++){
		make_holiday(arr_year[i], arr_month[i], holiday_name[i], holiday_num[i], &hol_max);		/* ファイルcalendar_main.c */
	}
	print_calendar_three_gengoh(arr_year, arr_month);													/* ファイルoutput_calendar_gengoh.c */
	row_calendar_main_today(cal_buf, week_max, holiday_num, today.month, today.day, arr_month);	/* カレンダー表示(ファイルoutput_calendar.c) */
	row_holiday(holiday_name, hol_max);																	/* 祝日を表示(ファイルoutput_calendar.c) */
}

