/*!
 * @file
 * @brief カレンダーを出力するプログラム
 * @author y_okamon
 * @version 1.0(2007.04.25)
 * @version 1.01(2008.02.19) <br>
 * 関数this_fiscal_cal内 1月〜3月はyear-1年度になるように修正
 * @version 1.1(2008.10.14)<br>
 * 土曜は青、日祝日は赤で表示するための仕様変更について<br>
 * (1)関数month_print_calendar、関数row_calendar_main 土日祝日に色を付けられるように変更<br>
 * (2)関数month_print_calendarと関数input_ym_theeeに祝日日付を入れる配列を新に設定<br>
 * <br>
 * カレンダーを横に3個表示するときの仕様変更<br>
 * (1) 3ヶ月分の年月を配列に入れたことにより、関数print_three_calendar内、ループで処理し行数を削減<br>
 * (2) 関数row_calendar_main内、型変更に伴う処理コードの書き換え<br>
 * (3) 関数row_calendar_mainが参照される関数を関数print_three_calendarから関数input_ym_threeに変更<br>
 * (4) (3)から関数print_three_calendarの仮引数の数の削減に成功<br>
 * <br>
 * その他の仕様変更<br>
 * (1) カレンダー操作部分をファイルopreate_calendar.cに移行<br>
 * (2) 関数row_calendar_main、関数row_calendar_main_todayから祝日表示部分を関数row_holidayに移行<br>
 * <br>
 */

/* ここからヘッダファイルの宣言 */
#include<stdio.h>
#include<string.h>
#include<time.h>
#include"number_set.h"

/* ここから関数のプロトタイプの宣言 */
/* ファイルcalendar_main.cから */
int day_of_week(int year, int month, int day);
int month_days(int year, int month);

/* ファイルinsert_holiday.cから */
int judge_month(int year, int month, char holiday_name[HOLY_MAX][HOLY_NAME], int holiday_days[HOLY_MAX]);

/*!
 * <b> 機能 </b><br>
 * year年month月のカレンダーを祝日とともに表示する<br>
 * <br>
 *
 * <b> 処理詳細　</b><br>
 * (1) year年month月の初日の曜日を算出<br>
 * (2) year年month月の月日数を取得<br>
 * (3) 初日から月末までのカレンダーを表示<br>
 * (3.1) 日曜日なら赤で表示<br>
 * (3.2) 土曜日なら青で表示<br>
 * (3.3) それ以外の曜日は黒で表示<br>
 * (4) 祝日名を表示する<br>
 * <br>
 * 
 * @param year 年
 * @param month 月
 * 
 * <b> 被参照自作関数 </b><br>
 * ファイルopreate_calendar.c内  関数next_month_cal<br>
 * ファイルopreate_calendar.c内  関数before_month_cal<br>
 * ファイルghcal.c内　関数argcheck<br>
 * <br>
 * 
 * <b> 参照自作関数 </b><br>
 * ファイルcalendar_main.c内  関数day_of_week<br>
 * ファイルcalendar_main.c内  関数month_days<br>
 * ファイルinsert_holidays.c内  関数judge_month<br>
 * <br>  
 *
 */
void month_print_calendar(int year, int month) 
{
    /* ここから変数の宣言 */
	int i,j = 0; 																		/* ループ変数 */
	char holiday_name[HOLY_MAX][HOLY_NAME];										/* 祝日名を入れる配列 */
	int holiday_num[HOLY_MAX];														/* 祝日日付を入れる配列 */
	int m_holidays = judge_month(year, month, holiday_name, holiday_num);		/* 月別の祝日を代入(ファイルinsert_hoiliday.c) */
	int week_val = day_of_week(year, month, 1); 									/* year年month月1日の曜日(ファイルcalendar_main.c) */
	int mdays = month_days(year, month); 											/* year年month月の月日数(ファイルcalendar_main.c) */

    /* ここから処理コードの記述　*/
	printf(" \x1b[31m日 \x1b[30m月 火 水 木 金 \x1b[34m土 \x1b[30m \n");	/* 日曜は赤、土曜は青 */
	printf("%*s", 3 * week_val, "");										/* 1日より左側のスペースを表示する */

	for(i = 1; i <= mdays; i++){											/* 月末まで */
		if(week_val == 0){													/* 日曜日なら */
			printf("\x1b[31m%3d", i);										/* 赤を表示する */
			if(j != m_holidays && holiday_num[j] == i){					/* その日が祝日 */
				j++;															/* 次の祝日に移す */
			}
		}else if(week_val != 0 && holiday_num[j] == i){					/* 日曜でなく、祝日である */
			printf("\x1b[31m%3d", i);										/* その日は赤で表示 */
			j++;																/* 次の祝日へ */
		}else if(week_val != 0 && holiday_num[j] != i){					/* 日曜でなく、祝日でもない */
			if(week_val == 6){												/* 土曜日である */
				printf("\x1b[34m%3d", i);									/* その日を青で表示 */
			}else{																/* そうでなければ */
				printf("\x1b[30m%3d", i);									/* その日は黒で表示する */
			}
		}
		if(++week_val == 7){													/* 日曜日になったら */
			week_val = 0;														/* 曜日を0にする */
			printf("\n");														/* 改行する */
		}
	}

	if(day_of_week(year, month, mdays) != 6){								/* 月末が土曜日以外なら(ファイルcalendar_main.c) */
		printf("\x1b[30m\n\n");												/* 2行改行する */
	}else{																		/* 月末が土曜日である*/
		printf("\x1b[30m\n");												/* 1行改行する */
	}
	for(i = 0; i < m_holidays; i++) {										/* 祝日名を表示 */
		printf("%2s\n", holiday_name[i]);
	}
	if(m_holidays != 0){														/* 祝日が1日もないときは改行 */
		printf("\n");
	}
}

/*!
 * <b> 機能 </b><br>
 * year年month月のカレンダーを祝日とともに表示し、システム上の日付の日を白抜きにする<br>
 * <br>
 *
 * <b> 処理詳細　</b><br>
 * (1) year年month月の初日の曜日を算出<br>
 * (2) year年month月の月日数を取得<br>
 * (3) 初日から月末までのカレンダーを表示<br>
 * (3.1) 日曜日なら赤で表示<br>
 * (3.2) 土曜日なら青で表示<br>
 * (3.3) それ以外の曜日は黒で表示<br>
 * (3.4) システム上の日付なら文字色と背景色を反転<br>
 * (4) 祝日名を表示する<br>
 * <br>
 * 
 * @param year 年
 * @param month 月
 * @param day 日
 * 
 * <b> 被参照自作関数 </b><br>
 * ファイルopreate_calendar.c内  関数this_month_cal<br>
 * <br>
 * 
 * <b> 参照自作関数 </b><br>
 * ファイルcalendar_main.c内  関数day_of_week<br>
 * ファイルcalendar_main.c内  関数month_days<br>
 * <br>  
 *
 */
void month_print_calendar_today(int year, int month, int day) 
{
    /* ここから変数の宣言 */
	int i,j = 0;																	/* ループ変数 */
	char holiday_name[HOLY_MAX][HOLY_NAME];									/* 祝日名を入れる配列 */
	int holiday_num[HOLY_MAX];													/* 祝日日付を入れる配列 */
	int m_holidays = judge_month(year, month, holiday_name, holiday_num);	/* 月別の祝日を代入(ファイルinsert_holiday.c) */
	int week_val = day_of_week(year, month, 1);								/* year年month月1日の曜日(ファイルcalendar_main.c) */
	int mdays = month_days(year, month);										/* year年month月の月日数(ファイルcalendar_main.c) */

    /* ここから処理コードの記述　*/
	printf(" \x1b[31m日 \x1b[30m月 火 水 木 金 \x1b[34m土 \x1b[0m \n");	/* 日曜は赤、土曜は青 */
	printf("%*s", 3 * week_val, "");										/* 1日より左側のスペースを表示する */

	for(i = 1; i <= mdays; i++){											/* 月末まで */
		if(week_val == 0){													/* 日曜日なら */
			if(i == day){														/* その日が今日 */
				printf(" \x1b[31m\x1b[7m%2d\x1b[0m", i);					/* その日の文字色と背景色を反転表示 */
			}else{																/* その日が今日でない */
				printf("\x1b[31m%3d\x1b[0m", i);							/* その日を赤文字で表示 */
			}
			if(j != m_holidays && holiday_num[j] == i){					/* その日が祝日 */
				j++;															/* 次の祝日に移す */
			}
		}else if (week_val != 0 && holiday_num[j] == i){					/* 日曜でなく、祝日である */
			if(i == day){														/* その日が今日 */
				printf(" \x1b[31m\x1b[7m%2d\x1b[0m", i);					/* 文字色と背景色を反転 */
			}else{																/* その日が今日でない */
				printf("\x1b[31m%3d\x1b[0m", i);							/* その日は赤文字表示 */
			}
			j++;																/* 次の祝日へ */
		}else if (week_val != 0 && holiday_num[j] != i){					/* 日曜でなく、祝日でもない */
			if(week_val == 6){												/* 土曜日である */
				if(i == day){													/* その日が今日 */
					printf(" \x1b[34m\x1b[7m%2d\x1b[0m", i);				/* その日の文字色と背景色を反転 */
				}else{															/* その日が今日でない */
					printf("\x1b[34m%3d\x1b[0m", i);						/* その日を青で表示 */
				}
			}else{																/* そうでなければ */
				if(i == day){													/* その日が今日 */
					printf(" \x1b[7m%2d\x1b[0m",i);							/* 文字色と背景色を反転 */
				}else{
					printf("%3d", i);											/* その日は黒で表示する */
				}
			}
		}
		if(++week_val == 7){													/* 日曜日になったら */
			week_val = 0;														/* 曜日を0にする */
			printf("\n");														/* 改行する */
		}
	}

	if(day_of_week(year, month, mdays) != 6){								/* 月末が土曜日以外なら */
        printf("\n\n");														/* 2行改行する */
	}else{																		/* 月末が土曜日である*/
		printf("\n");															/* 1行改行する */
	}
	for(i = 0; i < m_holidays; i++){										/* 祝日名を表示 */
		printf("%2s\n", holiday_name[i]);
	}
	if(m_holidays != 0){														/* 祝日が1日もないときは改行 */
		printf("\n");
	}
}

/*!
 * <b> 機能</b>
 * 指定された3ヶ月分の年と月を表示する<br>
 * <br>
 *
 * <b> 処理詳細</b><br>
 * (1) 指定された3ヶ月分の年と月をバッファに入れる<br>
 * (2) バッファを横に表示する<br>
 * <br>
 *
 * @param year[] 年
 * @param month[] 月
 *
 * <b> 被参照自作関数</b><br>
 * ファイルopreate_calendar.c  関数input_ym_three<br>
 * ファイルorpeate_calendar.c  関数input_ym_three_today<br>
 * <br>
 * 
 * <b> 参照自作関数</b><br>
 * なし<br>
 * <br>
 * 
 */
void print_calendar_three(int year[COL_CAL], int month[COL_CAL]) 
{
    /* ここから変数の宣言 */
	int i;								/* ループ変数 */
	char year_month[COL_CAL][30];	/* 表示のための変数 */
    
    /* ここから処理コードの記述 */
	for(i = 0; i < 3; i++){
		sprintf(year_month[i], "   %8d年%2d月     ", year[i], month[i]);	/* 年月を表示するための準備 */
		printf("%s ", year_month[i]);
	}
	printf("\n");
}

/*!
 * <b> 機能 </b><br>
 * カレンダーを横に表示させる <br>
 * <br>
 *
 * <b> 処理詳細</b><br>
 * (1) 曜日を表示<br>
 * (2) 日付を曜日ごと、祝日ごとに色分けする<br>
 * <br>
 * 
 * @param cal_buf[][][] カレンダーバッファ
 * @param week_max 最大週数
 * @param holiday_num[][] 日付代入配列
 * 
 * <b> 被参照自作関数 </b><br>
 * ファイルopreate_calendar.c内  関数input_ym_three<br>
 * ファイルopreate_calendar.c内  関数input_ym_three_gengoh<br>
 * ファイルopreate_calendar_gengoh.c内　関数input_1989_1to3<br>
 * ファイルopreate_calendar_gengoh.c内  関数input_1989_three<br>
 * <br>
 *
 * <b> 参照自作関数</b><br>
 * なし<br>
 * <br>
 *
 */
void row_calendar_main(int cal_buf[COL_CAL][WEEK_ROW][WEEK_COL], int week_max, int holiday_num[COL_CAL][HOLY_MAX]) 
{
    /* ここから変数の宣言 */
	int i, j, k;					/* ループ変数 */
	int index[3] = {0, 0, 0};	/* 祝日日付代入配列の要素数 */

    /* ここから処理コードの記述 */
	for(i = 0; i < COL_CAL; i++){											/* 曜日を表示する */
		printf(" \x1b[31m日 \x1b[30m月 火 水 木 金 \x1b[34m土\x1b[m  ");	/* 日曜は赤、土曜は青 */
	}
	printf("\n");
    
	for(i = 0; i < week_max; i++){											/* 週を5か6個 */
		for(j = 0; j < COL_CAL; j++){										/* 横に3個ずつ */
			if(cal_buf[j][i][0] == 0){										/* 日曜が空白 */
				printf("   ");												/* 3文字分空白 */
			}else{																/* 空白ではない */
				if(cal_buf[j][i][0] == holiday_num[j][index[j]]){		/* その日が祝日 */
					index[j]++;												/* 指定された月の次の要素に行く */
				}
				printf("\x1b[31m%3d", cal_buf[j][i][0]);					/* 赤で表示 */
			}
			for(k = 1; k < 6; k++){											/* 平日 */
				if(cal_buf[j][i][k] == 0){									/* 平日が空白 */
					printf("   ");											/* 3文字分空白 */
				}else{ 														/* そうでない */
					if(cal_buf[j][i][k] == holiday_num[j][index[j]]){	/* その日が祝日 */
						printf("\x1b[31m%3d", cal_buf[j][i][k]);			/* 赤で表示 */
						index[j]++;											/* 指定された月の次の要素に行く */
					}else{														/* そうでない */
						printf("\x1b[30m%3d", cal_buf[j][i][k]);			/* 黒で表示 */
					}
				}
			}
			if(cal_buf[j][i][6] == 0){										/* 土曜日が空白 */
				printf("   ");												/* 3文字分空白 */
			}else{																/* そうでない */
				if(cal_buf[j][i][6] == holiday_num[j][index[j]]){		/* その日が祝日 */
					printf("\x1b[31m%3d", cal_buf[j][i][6]);				/* 赤で表示 */
					index[j]++;												/* 指定された月の次の要素に行く */
				}else{															/* そうでない */
					printf("\x1b[34m%3d", cal_buf[j][i][6]);				/* 青で表示 */
				}
			}
			printf("  ");														/* 2文字分空白 */
		}
		printf("\x1b[30m\n");                                             
	}
}

/*!
 * <b> 機能 </b><br>
 * カレンダーを横に表示させる <br>
 * <br>
 *
 * <b> 処理詳細</b><br>
 * (1) 曜日を表示<br>
 * (2) 日付を曜日ごと、祝日ごとに色分けする<br>
 * (3) システム上の日付を文字色と背景色と反転<br>
 * (4) 祝日を表示する<br>
 * <br>
 * 
 * @param cal_buf[][][] カレンダーバッファ
 * @param week_max 最大週数
 * @param holiday_num[][] 日付代入配列
 * @param month システム上の月
 * @param day システム上の年
 * @param arr_month 3ヶ月分の月
 * 
 * <b> 被参照自作関数 </b><br>
 * ファイルorpeate_calendar.c内  関数input_ym_three_today<br>
 * ファイルopreate_calendar_gengoh.c内  関数input_ym_three_today_gengoh<br>
 * <br>
 *
 * <b> 参照自作関数</b><br>
 * なし<br>
 * <br>
 *
 */
void row_calendar_main_today(int cal_buf[COL_CAL][WEEK_ROW][WEEK_COL], int week_max, int holiday_num[COL_CAL][HOLY_MAX], int month, int day, int arr_month[COL_CAL]) 
{
	/* ここから変数の宣言 */
	int i, j, k;               /* ループ変数 */
	int index[3] = {0, 0, 0};  /* 祝日日付代入配列の要素数 */

	/* ここから処理コードの記述 */
	for (i = 0; i < COL_CAL; i++){															/* 曜日を表示する */
		printf(" \x1b[31m日 \x1b[30m月 火 水 木 金 \x1b[34m土\x1b[0m  ");					/* 日曜は赤、土曜は青 */
	}
	printf("\n");
    
	for(i = 0; i < week_max; i++){															/* 週を5か6個 */
		for(j = 0; j < COL_CAL; j++){														/* 横に3個ずつ */
			if(cal_buf[j][i][0] == 0){														/* 日曜が空白 */
				printf("   ");																/* 3文字分空白 */
			}else{																				/* 空白ではない */
				if(cal_buf[j][i][0] == holiday_num[j][index[j]]){						/* その日が祝日 */
					index[j]++;																/* 指定された月の次の要素に行く */
				}
				if(arr_month[j] == month && cal_buf[j][i][0] == day){					/* その日が今日 */
					printf(" \x1b[31m\x1b[7m%2d\x1b[0m", cal_buf[j][i][0]);				/* 文字色と背景色を反転させる */
				}else{																			/* そうでない */
					printf("\x1b[31m%3d\x1b[0m", cal_buf[j][i][0]);						/* 赤で表示 */
				}
			}
			for(k = 1; k < 6; k++){															/* 平日 */
				if(cal_buf[j][i][k] == 0){													/* 平日が空白 */
					printf("   ");															/* 3文字分空白 */
				}else{																			/* そうでない */
					if(cal_buf[j][i][k] == holiday_num[j][index[j]]){					/* その日が祝日 */
						if(arr_month[j] == month && cal_buf[j][i][k] == day){			/* その日が今日である */
							printf(" \x1b[31m\x1b[7m%2d\x1b[0m", cal_buf[j][i][k]);		/* 文字色と背景色を反転させる */
						}else{																	/* そうでない */
							printf("\x1b[31m%3d\x1b[0m", cal_buf[j][i][k]);				/* 赤で表示 */
						}
						index[j]++;															/* 指定された月の次の要素に行く */
					}else{																		/* そうでない */
						if(arr_month[j] == month && cal_buf[j][i][k] == day){			/* その日が今日 */
							printf(" \x1b[7m%2d\x1b[0m", cal_buf[j][i][k]);				/* 文字色と背景色を反転 */
						}else{																	/* そうでない */
							printf("%3d", cal_buf[j][i][k]);								/* 黒で表示 */
						}
					}
				}
			}
			if(cal_buf[j][i][6] == 0){														/* 土曜日が空白 */
				printf("   ");																/* 3文字分空白 */
			}else{																				/* そうでない */
				if(cal_buf[j][i][6] == holiday_num[j][index[j]]){						/* その日が祝日 */
					if(arr_month[j] == month && cal_buf[j][i][6] == day){				/* その日が今日 */ 
						printf(" \x1b[31m\x1b[7m%2d\x1b[0m", cal_buf[j][i][6]);			/* 文字色と背景色を反転 */
					}else{																		/* そうでない */
						printf("\x1b[31m%3d\x1b[0m", cal_buf[j][i][6]);					/* 赤で表示 */
					}
					index[j]++;																/* 指定された月の次の要素に行く */
				}else{																			/* そうでない */
					if(arr_month[j] == month && cal_buf[j][i][6] == day){				/* その日が土曜 */
						printf(" \x1b[34m\x1b[7m%2d\x1b[0m", cal_buf[j][i][6]);			/* 文字色と背景色を反転 */
					}else{																		/* そうでない */
						printf("\x1b[34m%3d\x1b[0m", cal_buf[j][i][6]);					/* 青で表示 */
					}
				}
			}
			printf("  ");																		/* 2文字分空白 */
		}
		printf("\n");                                          	
	}
}

/*!
 * <b> 機能</b><br> 
 * 3ヶ月分の祝日を横に表示させる <br>
 * 
 * @param holiday_name[][][] 祝日
 * @param hol_max 最大祝日日数
 * 
 * <b> 被参照自作関数 </b><br>
 * ファイルopreate_calendar.c内 関数input_ym_three<br>
 * ファイルopreate_calendar.c内 関数input_ym_three_today<br>
 * <br>
 *
 * <b> 参照自作関数</b><br>
 * なし<br>
 * <br>
 */
void row_holiday(char holiday_name[COL_CAL][HOLY_MAX][HOLY_NAME], int hol_max)
{
	/* ここから変数の宣言 */
	int i, j;     /* ループ変数 */
    
	/* ここから処理コードの記述 */
	printf("\n");
	for (i = 0; i < hol_max; i++) {                     /* 祝日を */
		for (j = 0; j < COL_CAL; j++) {                 /* 横に3個並べて */
			printf("%s  ", holiday_name[j][i]);         /* 表示する */
		}
		printf("\n");
	}
	printf("\n");
}
