/*!
 * @file
 * @brief カレンダーを出力するプログラム
 * @author y_okamon
 * @version 2.0(2011.07.01)<br>
 * 新ファイル作成の意図<br>
 * 年表記を元号でも表記できるようにする。
 */

/* ここからヘッダファイルの宣言 */
#include<stdio.h>
#include<string.h>
#include<time.h>
#include"number_set.h"

/* ここから関数のプロトタイプの宣言 */
/* ファイルcalendar_main.cから */
int day_of_week(int year, int month, int day);
int month_days(int year, int month);

/* ファイルinsert_holiday.cから */
int judge_month(int year, int month, char holiday_name[HOLY_MAX][HOLY_NAME], int holiday_days[HOLY_MAX]);


/*!
 * <b> 機能 </b><br>
 * year年month月のカレンダーを祝日とともに表示する<br>
 * <br>
 *
 * <b> 処理詳細　</b><br>
 * (1) year年month月の初日の曜日を算出<br>
 * (2) year年month月の月日数を取得<br>
 * (3) 初日から月末までのカレンダーを表示<br>
 * (3.1) 日曜日なら赤で表示<br>
 * (3.2) 土曜日なら青で表示<br>
 * (3.3) それ以外の曜日は黒で表示<br>
 * (4) 祝日名を表示する<br>
 * <br>
 * 
 * @param year 年
 * @param month 月
 * 
 * <b> 被参照自作関数 </b><br>
 * ファイルopreate_calendar_gengoh.c内  関数opreate_year_month_gengoh<br>
 * <br>
 * 
 * <b> 参照自作関数 </b><br>
 * ファイルcalendar_main.c内  関数day_of_week<br>
 * ファイルcalendar_main.c内  関数month_days<br>
 * ファイルinsert_holidays.c内　関数judge_month<br>
 * <br>  
 *
 */
void month_print_calendar_s64_1(int year, int month) 
{
	/* ここから変数の宣言 */
	int i,j = 0; 																		/* ループ変数 */
	char holiday_name[HOLY_MAX][HOLY_NAME];										/* 祝日名を入れる配列 */
	int holiday_num[HOLY_MAX];														/* 祝日日付を入れる配列 */
	int m_holidays = judge_month(year, month, holiday_name, holiday_num);	/* 月別の祝日を代入(ファイルinsert_holidays.c) */
	int week_val = day_of_week(year, month, 1); 									/* year年month月1日の曜日(ファイルcalendar_main.c) */
	int heisei;
	int syowa;

	/* ここから処理コードの記述　*/
	printf(" \x1b[31m日 \x1b[30m月 火 水 木 金 \x1b[34m土 \x1b[30m \n");		/* 日曜は赤、土曜は青 */
	printf("%*s", 3 * week_val, "");											/* 1日より左側のスペースを表示する */

	for(i = 1; i <= 7; i++){													/* 月末まで */
		if(week_val == 0){														/* 日曜日なら */
			printf("\x1b[31m%3d", i);											/* 赤を表示する */
			if(j != m_holidays && holiday_num[j] == i){						/* その日が祝日 */
				j++;																/* 次の祝日に移す */
			}
		}else if(week_val != 0 && holiday_num[j] == i){						/* 日曜でなく、祝日である */
			printf("\x1b[31m%3d", i);											/* その日は赤で表示 */
			j++;																	/* 次の祝日へ */
		}else if(week_val != 0 && holiday_num[j] != i){						/* 日曜でなく、祝日でもない */
			if(week_val == 6){													/* 土曜日である */
				printf("\x1b[34m%3d", i);										/* その日を青で表示 */
			}else{																	/* そうでなければ */
				printf("\x1b[30m%3d", i);										/* その日は黒で表示する */
			}
		}
		if(++week_val == 7){														/* 日曜日になったら */
			week_val = 0;															/* 曜日を0にする */
			printf("\n");															/* 改行する */
		}
	}

	if(day_of_week(year, month, 7) != 6){										/* 月末が土曜日以外なら(ファイルcalendar_main.c) */
		printf("\x1b[30m\n\n");													/* 2行改行する */
	}else{																			/* 月末が土曜日である*/
		printf("\x1b[30m\n");													/* 1行改行する */
	}
	for(i = 0; i < 2; i++) {													/* 祝日名を表示 */
		printf("%2s\n", holiday_name[i]);
	}
	if(m_holidays != 0){															/* 祝日が1日もないときは改行 */
		printf("\n");
	}
}

/*!
 * <b> 機能 </b><br>
 * year年month月のカレンダーを祝日とともに表示する<br>
 * <br>
 *
 * <b> 処理詳細　</b><br>
 * (1) year年month月の初日の曜日を算出<br>
 * (2) year年month月の月日数を取得<br>
 * (3) 初日から月末までのカレンダーを表示<br>
 * (3.1) 日曜日なら赤で表示<br>
 * (3.2) 土曜日なら青で表示<br>
 * (3.3) それ以外の曜日は黒で表示<br>
 * (4) 祝日名を表示する<br>
 * <br>
 * 
 * @param year 年
 * @param month 月
 * 
 * <b> 被参照自作関数 </b><br>
 * ファイルopreate_calendar_gengoh.c内  関数opreate_year_month_gengoh<br>
 * <br>
 * 
 * <b> 参照自作関数 </b><br>
 * ファイルcalendar_main.c内  関数day_of_week<br>
 * ファイルcalendar_main.c内  関数month_days<br>
 * ファイルinsert_hoildays.c内 関数judge_months<br>
 * <br>  
 *
 */
void month_print_calendar_h1_1(int year, int month) 
{
    /* ここから変数の宣言 */
	int i,j = 0; 																		/* ループ変数 */
	char holiday_name[HOLY_MAX][HOLY_NAME];										/* 祝日名を入れる配列 */
	int holiday_num[HOLY_MAX];														/* 祝日日付を入れる配列 */
	int m_holidays = judge_month(year, month, holiday_name, holiday_num);	/* 月別の祝日を代入(ファイルinsert_holidays.c) */
	int week_val = day_of_week(year, month, 1); 									/* year年month月1日の曜日(ファイルcalendar_main.c) */
	int mdays = month_days(year, month); 											/* year年month月の月日数(ファイルcalendar_main.c) */
	int heisei;
	int syowa;

    /* ここから処理コードの記述　*/
	printf(" \x1b[31m日 \x1b[30m月 火 水 木 金 \x1b[34m土 \x1b[30m \n");		/* 日曜は赤、土曜は青 */
	printf("\n");
	printf("%*s", 3 * week_val, "");											/* 1日より左側のスペースを表示する */

	for(i = 8; i <= mdays; i++){												/* 月末まで */
		if(week_val == 0){														/* 日曜日なら */
			printf("\x1b[31m%3d", i);											/* 赤を表示する */
			if(j != m_holidays && holiday_num[j] == i){						/* その日が祝日 */
				j++;																/* 次の祝日に移す */
			}
		}else if(week_val != 0 && holiday_num[j] == i){						/* 日曜でなく、祝日である */
			printf("\x1b[31m%3d", i);											/* その日は赤で表示 */
			j++;																	/* 次の祝日へ */
		}else if(week_val != 0 && holiday_num[j] != i){						/* 日曜でなく、祝日でもない */
			if(week_val == 6){													/* 土曜日である */
				printf("\x1b[34m%3d", i);										/* その日を青で表示 */
			}else{																	/* そうでなければ */
				printf("\x1b[30m%3d", i);										/* その日は黒で表示する */
			}
		}
		if(++week_val == 7){														/* 日曜日になったら */
			week_val = 0;															/* 曜日を0にする */
			printf("\n");															/* 改行する */
		}
	}

	if(day_of_week(year, month, mdays) != 6){									/* 月末が土曜日以外なら(ファイルcalendar_main.c) */
		printf("\x1b[30m\n\n");													/* 2行改行する */
	}else{																			/* 月末が土曜日である*/
		printf("\x1b[30m\n");													/* 1行改行する */
	}
	for(i = 2; i < 4; i++) {													/* 祝日名を表示 */
		printf("%2s\n", holiday_name[i]);
	}
	if(m_holidays != 0){															/* 祝日が1日もないときは改行 */
		printf("\n");
	}
}

/*!
 * <b> 機能</b>
 * 指定された3ヶ月分の年と月を表示する(元号版)<br>
 * <br>
 *
 * <b> 処理詳細</b><br>
 * (1) 指定された3ヶ月分の年と月をバッファに入れる<br>
 * (2) バッファを横に表示する<br>
 * <br>
 *
 * @param year[] 年
 * @param month[] 月
 *
 * <b> 被参照自作関数</b><br>
 * ファイルopreate_calendar_gengoh.c内  関数input_ym_three_gengoh<br>
 * ファイルopreate_calendar_gengoh.c内　関数input_ym_three_today_gengoh<br>
 * <br>
 * 
 * <b> 参照自作関数</b><br>
 * なし<br>
 * <br>
 * 
 */
void print_calendar_three_gengoh(int year[COL_CAL], int month[COL_CAL]) 
{
    /* ここから変数の宣言 */
	int i;								/* ループ変数 */
	char year_month[COL_CAL][30];	/* 表示のための変数 */
    
    /* ここから処理コードの記述 */
	if(year[0] > 1989){
		for(i = 0; i < 3; i++){
			sprintf(year_month[i], "     平成%2d年%2d月     ", year[i] - 1988, month[i]);	/* 年月を表示するための準備 */
			printf("%s ", year_month[i]);
		}
	}else if(year[0] < 1989){
		for(i = 0; i < 3; i++){
			sprintf(year_month[i], "     昭和%2d年%2d月     ", year[i] - 1925, month[i]);	/* 年月を表示するための準備 */
			printf("%s ", year_month[i]);
		}
	}
	printf("\n");
}

/*!
 * <b> 機能</b>
 * 指定された3ヶ月分の年と月を表示する(平成元年版)<br>
 * <br>
 *
 * <b> 処理詳細</b><br>
 * (1) 指定された3ヶ月分の年と月をバッファに入れる<br>
 * (2) バッファを横に表示する<br>
 * <br>
 *
 * @param year[] 年
 * @param month[] 月
 *
 * <b> 被参照自作関数</b><br>
 * ファイルopreate_calendar.c内  関数input_1989_three<br>
 * ファイルopreate_calendar_gengoh.c内　関数input_1989_1to3<br>
 * <br>
 * 
 * <b> 参照自作関数</b><br>
 * なし<br>
 * <br>
 * 
 */
void print_calendar_three_1989(int year[COL_CAL], int month[COL_CAL]) 
{
    /* ここから変数の宣言 */
	int i;								/* ループ変数 */
	char year_month[COL_CAL][30];	/* 表示のための変数 */
    
    /* ここから処理コードの記述 */
	for(i = 0; i < 3; i++){
		sprintf(year_month[i], "     平成元年%2d月     ", month[i]);	/* 年月を表示するための準備 */
		printf("%s ", year_month[i]);
	}
	printf("\n");
}

